﻿using Microsoft.Xna.Framework;
using UWBGL_XNA_Lib13;
using System.Collections.Generic;
using NathanScott_Lib;
using System;
using System.Xml.Serialization;
using System.Windows.Forms;

namespace FinalProject
{
    [Serializable]
    public class Hero : SceneObject
    {
        //Stats
        private float mMaxDiveDepth = 0.0f;
        private TimeSpan mDiveTime = new TimeSpan();
        private float mAirLossRate = 0.0f;

        private Inventory mInventory = new Inventory();
        private Inventory mBag = new Inventory();
        private int mBagSize = 5;
        private float mMoney = 0.00f;
        private bool mIsAlive = true; // default
        private Timer mDeathTimer = null;
        private bool mIsDoneDying = false; //default

        //This is only temporary until you get a tank of some sort
        private float mAir = 25.0f;
        private float mAirMax = 25.0f;
        private AirTank mAirtank = null;
        private float mRightArmRotationDegrees;
        private Item mEquippedItem;

        const float Normal_DragFactor = 0.85f;

        private bool mStopMovingDown = false;
        private bool mStopMovingUp = false;
        private bool mStopMovingLeft = false;
        private bool mStopMovingRight = false;

        private bool mWon = false;

        // for xml serialization
        public string name;
        public Vector3 location;
        public float torsoWidth;
        public float torsoHeight;
        public float armLength;
        public float armWidth;
        public float legLength;
        public float legWidth;
        public float headRadius;

        public Hero()
            : base()
        {

        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Hero"/> class.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="location">The location.</param>
        /// <param name="torsoWidth">Width of the torso.</param>
        /// <param name="torsoHeight">Height of the torso.</param>
        /// <param name="armLength">Length of the arm.</param>
        /// <param name="armWidth">Width of the arm.</param>
        /// <param name="legLength">Length of the leg.</param>
        /// <param name="legWidth">Width of the leg.</param>
        /// <param name="headRadius">The head radius.</param>
        public Hero(string name, Vector3 location, float torsoWidth, float torsoHeight, float armLength, float armWidth, float legLength, float legWidth, float headRadius):base(name)
        {
            this.name = name;
            this.location = location;
            this.torsoWidth = torsoWidth;
            this.torsoHeight = torsoHeight;
            this.armLength = armLength;
            this.armWidth = armWidth;
            this.legLength = legLength;
            this.legWidth = legWidth;
            this.headRadius = headRadius;

            mWon = false;

            // we want to recognize this guy as the hero.
            this.mObjType = eSceneObjectType.Hero;

            //Set the hero's location
            mXFormInfo.setPivot(location);

            UWB_SceneNode torso = new Torso(location, torsoWidth, torsoHeight, "Torso");
            insertChildNode(torso);

            //put the leg half way between the torso width
            Vector3 legPos = location;
            legPos.X -= torsoHeight / 2;

            //put the arm half way between the torso width
            Vector3 armPos = location;
            armPos.X += torsoHeight / 4;

            Vector3 headPos = location;
            headPos.X += torsoHeight - (headRadius / 2);

            UWB_SceneNode leftArm = new Arm(armPos, armLength, armWidth, "Left Arm");
            torso.insertChildNode(leftArm);
            UWB_SceneNode leftLeg = new Leg(legPos, legLength, legWidth, "Left Leg");
            torso.insertChildNode(leftLeg);

            UWB_SceneNode head = new Head(headPos, headRadius, "Head");
            torso.insertChildNode(head);

            UWB_SceneNode rightLeg = new Leg(legPos, legLength, legWidth, "Right Leg");
            torso.insertChildNode(rightLeg);

            UWB_SceneNode rightArm = new Arm(armPos, armLength, armWidth, "Right Arm");
            torso.insertChildNode(rightArm);

            UWB_Manager.mAudioManager.playSound(eSoundFiles.UnderwaterScuba.ToString(), 0.2f, 0, 0, true);
        }

        public void updateWinGame()
        {
            if (Inventory.getItemOfSameType(typeof(WinGameItem)) != null)
                mWon = true;
        }

        public bool addInventoryItem(Item item)
        {
            if (mMoney >= item.Price)
            {
                mMoney -= item.Price;
                mInventory.addItem(item);

                //If this just happens to be the win game item set us to won
                if (item is WinGameItem)
                    mWon = true;

                //equip our new item if we don't have one equipped
                if (mEquippedItem == null && item.isEquippable())
                {
                    Item lastItem = null;
                    setEquippedItem(item, ref lastItem);
                }

                //If we added an air tank item then do something special
                if (item is AirTank)
                    changeAirTank(item as AirTank);

                return true;
            }

            return false;
        }

        /// <summary>
        /// Set the item to be equipped in the hero's right hand (un-equipping last item there)
        /// </summary>
        /// <param name="item">Item to be equipped</param>
        /// <param name="lastItem">last equipped item (null if none)</param>
        /// <returns>successful or not</returns>
        public bool setEquippedItem(Item item, ref Item lastItem)
        {
            lastItem = null;

            UWB_SceneNode rightArm = findNode("Right Arm", true, true);
            if (rightArm != null)
            {
                UWB_SceneNode rightHand = rightArm.findNode("Hand", true, true);
                if (rightHand != null)
                {
                    //Assume first item in the hand
                    if (rightHand.numChildren() > 0)
                    {
                        UWB_SceneNode node = rightHand.getChildNode(0);
                        if (node is Item)
                        {
                            lastItem = node as Item;

                            rightHand.removeChildNode(node);
                        }
                    }

                    rightHand.insertChildNode(item);

                    mEquippedItem = item;

                    Program.updateEquippedItem(item);

                    return true;
                }
            }

            return false;
        }

        public SceneObject useEquippedItem()
        {
            if (mEquippedItem != null && mEquippedItem.canUseItem())
            {
                UWB_XFormInfo xFormToRightHand = GetSumOfXFormDownToRightHand();

                return mEquippedItem.useItem(xFormToRightHand);
            }

            return null;
        }

        // ONLY FOR USE WITH XML!
        [XmlElement("EquippedItem")]
        public Item EquippedItemXML
        {
            get
            {
                return this.mEquippedItem;
            }
            set
            {
                this.mEquippedItem = value;
            }
        }

        /// <summary>
        /// Get the item currently equipped to the right hand
        /// </summary>
        /// <returns>item currently equipped to the right hand, null if none or error</returns>
        public Item getEquippedItem()
        {
            UWB_SceneNode rightArm = findNode("Right Arm", true, true);
            if (rightArm != null)
            {
                UWB_SceneNode rightHand = rightArm.findNode("Hand", true, true);
                if (rightHand != null)
                {
                    //Assume first item in the hand
                    if (rightHand.numChildren() > 0)
                    {
                        UWB_SceneNode node = rightHand.getChildNode(0);
                        if (node is Item)
                        {
                            return node as Item;
                        }
                    }
                }
            }

            return null;
        }


        public void changeAirTank(AirTank airTank)
        {
            //If we already have an air tank remove it since we have a new one
            if (mAirtank != null)
            {
                mInventory.removeItem(mAirtank);
                removeChildNode(mAirtank);
            }

            UWB_SceneNode torso = findNode("Torso");
            if (torso != null)
            {
                airTank.getXFormInfo().setTranslation(torso.getXFormInfo().getPivot());
            }

            insertChildNode(airTank);
            mAirtank = airTank;
        }

        public bool subtractMoney(float amount)
        {
            if (mMoney >= amount)
            {
                mMoney -= amount;
                return true;
            }

            return false;
        }

        public int InventoryCount
        {
            get
            {
                return mInventory.getItemCount();
            }
        }

        public Inventory Inventory
        {
            get
            {
                return mInventory;
            }
            set
            {
                mInventory = value;
            }
        }

        public Item getInventoryItem(int index)
        {
            return mInventory.getItem(index);
        }

        public bool hasAirTank()
        {
            return this.mAirtank != null;
        }

        public Item getAirTank()
        {
            return mAirtank;
        }

        public bool addCollectible(Item item)
        {
            if (mBag.getItemCount() < mBagSize)
            {
                mBag.addItem(item);
                return true;
            }

            return false;
        }

        public bool sellCollectible(Item collectible)
        {
            if (mBag.removeItem(collectible))
            {
                mMoney += collectible.Price;
                return true;
            }

            return false;
        }

        public float Money
        {
            get
            {
                return mMoney;
            }
            set
            {
                mMoney = value;
            }
        }

        public Inventory Bag
        {
            get
            {
                return mBag;
            }
            set
            {
                mBag = value;
            }
        }

        public int BagCount
        {
            get
            {
                return mBag.getItemCount();
            }
        }

        public int BagSize
        {
            get
            {
                return mBagSize;
            }
            set
            {
                mBagSize = value;
            }
        }

        public float Air
        {
            get
            {
                return mAirtank != null ? mAirtank.Air : mAir;
            }
            set
            {
                if (mAirtank == null)
                {
                    mAir = value;
                }
                else
                {
                    mAirtank.Air = value;
                }
            }
        }

        public void decreaseAir(float amount)
        {
            if (IsAlive)
            {
                if (mAirtank != null)
                {
                    if (mAirtank.decreaseAir(amount))
                        GoThroughDeathMotions();
                }
                else
                {
                    float value = mAir - amount;

                    mAir = N_Math.Clamp(value, 0, mAirMax);

                    if (mAir <= 0.0f)
                        GoThroughDeathMotions();
                }
            }
        }

        public float AirMax
        {
            get
            {
                return mAirtank != null ? mAirtank.AirMax : mAirMax;
            }
            set
            {
                if (mAirtank == null)
                {
                    mAirMax = value;
                }
                else
                {
                    mAirtank.AirMax = value;
                }
            }
        }

        /// <summary>
        /// Set the air loss rate
        /// </summary>
        /// <param name="rate">rate at which we are loosing air (amount per second)</param>
        public void setAirLossRate(float rate)
        {
            mAirLossRate = rate;
        }

        public float getDiveDepth()
        {
            float depth = Program.getModel().getWaterBounds().getMax().Y - Position.Y;

            if (IsAlive && depth > mMaxDiveDepth)
                mMaxDiveDepth = depth;

            return depth;
        }

        public float getMaxDiveDepth()
        {
            return mMaxDiveDepth;
        }

        public TimeSpan getDiveTime()
        {
            return mDiveTime;
        }

        public void resetDiveTime()
        {
            if(IsAlive)
                mDiveTime = TimeSpan.Zero;
        }

        public void resetAir()
        {
            if (IsAlive)
            {
                if (mAirtank == null)
                {
                    Air = AirMax;
                }
            }
        }

        public TimeSpan getDiveTimeLeft()
        {
            if (mAirLossRate == 0.0f)
                return TimeSpan.Zero;

            double secondsLeft = Convert.ToDouble(Air) / Convert.ToDouble(mAirLossRate);
            return TimeSpan.FromSeconds(secondsLeft);
        }

        public void addToDiveTime(TimeSpan timeSpan)
        {
            if (IsAlive)
                mDiveTime += timeSpan;
        }

        public float RightArmRotationDegrees
        {
            get
            {
                float output = 0;
                UWB_SceneNode rightArm = findNode("Right Arm", true, true);
                if (rightArm != null)
                {
                    output = rightArm.getXFormInfo().getRotationInDegrees();
                }

                return output;
            }
            set
            {
                UWB_SceneNode rightArm = findNode("Right Arm", true, true);
                if (rightArm != null)
                {
                    rightArm.getXFormInfo().setRotationDegrees(value);
                }

                mRightArmRotationDegrees = value;
            }
        }

        /// <summary>
        /// Gets the knife bounds.
        /// </summary>
        /// <returns></returns>
        public UWB_BoundingBox GetKnifeBounds()
        {
            UWB_BoundingBox output = null;

            UWB_SceneNode knife = findNode("Knife", true, true);

            if (knife != null)
            {
                getNodeBounds(knife, false, ref output, Program.getModel().getDrawHelper(), false);
            }

            return output;
        }

        /// <summary>
        /// Gets the sum of XForm down to right hand - used for interacting with items he is holding
        /// </summary>
        /// <returns></returns>
        public UWB_XFormInfo GetSumOfXFormDownToRightHand()
        {
            UWB_XFormInfo output = new UWB_XFormInfo(getXFormInfo());

            // get X scale, because if he is facing left, we need to subtract instead of add.
            float xScale = output.getScale().X;
            bool add = xScale > 0 ? true : false;

            UWB_SceneNode torso = findNode("Torso", true, true);
            if (torso != null)
            {
                AddOrSubtractRotation(output, torso.getXFormInfo(), add);
            }

            UWB_SceneNode rightArm = torso.findNode("Right Arm", true, true);
            if (rightArm != null)
            {
                AddOrSubtractRotation(output, rightArm.getXFormInfo(), add);
            }

            UWB_SceneNode rightHand = rightArm.findNode("Hand", true, true);
            if (rightHand != null)
            {
                AddOrSubtractRotation(output, rightHand.getXFormInfo(), add);
            }

            return output;
        }

        /// <summary>
        /// Determines whether this instance is alive.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if this instance is alive; otherwise, <c>false</c>.
        /// </returns>
        public bool IsAlive
        {
            get
            {
                return mIsAlive;
            }
            set
            {
                mIsAlive = value;
            }
        }

        public bool hasWon()
        {
            return mWon;
        }

        public bool moveRight(float speed)
        {
            if (mStopMovingRight)
                return false;

            mVelocity.X = speed;

            return true;
        }

        public bool moveLeft(float speed)
        {
            if (mStopMovingLeft)
                return false;

            mVelocity.X = -speed;

            return true;
        }

        public bool moveUp(float speed)
        {
            if (mStopMovingUp)
                return false;

            mVelocity.Y = speed;

            return true;
        }

        public bool moveDown(float speed)
        {
            if (mStopMovingDown)
                return false;

            mVelocity.Y = -speed;

            return true;
        }

        public void resetMoves()
        {
            mStopMovingDown = false;
            mStopMovingLeft = false;
            mStopMovingRight = false;
            mStopMovingUp = false;
        }

        /// <summary>
        /// Response to a collision with another SceneObject.
        /// </summary>
        /// <param name="other">The other SceneObject this one collided with.</param>
        /// <param name="collisionPosition">The collision position.</param>
        public override void CollisionResponse(SceneObject other, Vector3 collisionPosition)
        {
            base.CollisionResponse(other, collisionPosition);

            Model model = Program.getModel();

            switch (other.getSceneObjectType())
            {
                case eSceneObjectType.Deadly:

                    // Do all the stuff we need to when Scuba Dude dies
                    GoThroughDeathMotions();

                    // reverse other object velocity and X scale
                    Vector3 otherObjectVelocity = other.getVelocity();
                    other.setVelocity(new Vector3(otherObjectVelocity.X * -1, otherObjectVelocity.Y * -1, otherObjectVelocity.Z));

                    UWB_XFormInfo otherXForm = other.getXFormInfo();
                    Vector3 otherObjectScale = otherXForm.getScale();
                    otherXForm.setScale(new Vector3(otherObjectScale.X * -1, otherObjectScale.Y, otherObjectScale.Z));
                    other.setXFormInfo(otherXForm);
                    
                    break;
                case eSceneObjectType.Safe:
                    mVelocity.X *= -1f;
                    mVelocity.Y *= -1f;

                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// Moves the hero by velocity, and keeps him within the water bounds.
        /// Also applies some drag, so he slowly comes to a stop.
        /// </summary>
        /// <param name="elapsedSecond">The elapsed second.</param>
        public override void moveNodeByVelocity(float elapsedSecond)
        {
            Model model = Program.getModel();

            // adjust velocity for drag
            Vector3 heroVelocity = this.getVelocity();

            Vector3 slowedVelocity = new Vector3(heroVelocity.X * Normal_DragFactor, heroVelocity.Y * Normal_DragFactor, heroVelocity.Z);
            this.setVelocity(slowedVelocity);

            // get water bounds from model, and keep hero within that, adjusting velocity as needed
            UWB_BoundingBox waterBounds = model.getWaterBounds();
            UWB_BoundingBox heroBounds = model.getHeroBounds();

            ConstrainHeroWithinWaterBounds(heroBounds, waterBounds);

            base.moveNodeByVelocity(elapsedSecond);

            rotateWithVelocity();

            if(isMoving())
                N_Utility.echoToStatusArea("Hero Pos: " + Position.ToString() + " Rotation: " + Rotation.ToString() + " Direction: " + (Scale.X < 0 ? "Left" : "Right"));
        }

        /// <summary>
        /// Goes the through death motions.
        /// </summary>
        private void GoThroughDeathMotions()
        {
            // Scuba dude is now dead.  So sad.
            mIsAlive = false;

            // Scuba dude's body will fall downward...
            setVelocity(new Vector3(0, -50, 0));

            //Animate Scuba Dude
            playAnimation(eAnimations.Death);

            // play sound effect.
            UWB_Manager.mAudioManager.stopLoopingSound(eSoundFiles.UnderwaterScuba.ToString());

            UWB_Manager.mAudioManager.playSound(eSoundFiles.Dying.ToString(), 1, 0, 0);

            mDeathTimer = new Timer();
            mDeathTimer.Interval = 3000;
            mDeathTimer.Tick += new EventHandler(mDeathTimer_Tick);
            mDeathTimer.Start();
        }

        void mDeathTimer_Tick(object sender, EventArgs e)
        {
            mIsDoneDying = true;
        }

        public bool IsDoneDying()
        {
            return mIsDoneDying;
        }

        /// <summary>
        /// Constrains the hero within bounds.
        /// </summary>
        /// <param name="heroBounds">The hero bounds.</param>
        /// <param name="waterBounds">The containing bounds.</param>
        private void ConstrainHeroWithinWaterBounds(UWB_BoundingBox heroBounds, UWB_BoundingBox waterBounds)
        {
            Vector3 newVelocity = this.getVelocity();

            float minX = heroBounds.getMin().X;
            float maxX = heroBounds.getMax().X;
            float minY = heroBounds.getMin().Y;
            float maxY = heroBounds.getMax().Y;

            // if minX is <= containing bounds min X (off screen to the left)
            if (minX <= waterBounds.getMin().X)
            {
                // We want to make sure the X velocity is positive.
                if (newVelocity.X < 0)
                {
                    newVelocity.X *= -1; // if negative, reverse it.
                }
            }

            // if maxX is >= containing bounds max X (off screen to the right)
            if (maxX >= waterBounds.getMax().X)
            {
                // We want to make sure the X velocity is negative.
                if (newVelocity.X > 0)
                {
                    newVelocity.X *= -1; // if negative, reverse it.
                }
            }

            // if minY is <= containing bounds min Y (off screen to the bottom)
            if (minY <= waterBounds.getMin().Y)
            {
                // We want to make sure the Y velocity is positive.
                if (newVelocity.Y < 0)
                {
                    float min = waterBounds.getMin().Y;
                    float max = waterBounds.getMin().Y + 10;
                    float value = maxY / max;
                    float drag = N_Math.fitToRange(value, Normal_DragFactor, 0.0f);
                    newVelocity.Y *= drag;

                    if (newVelocity.Y < 0)
                    {
                        newVelocity.Y *= -1; // if negative, reverse it.
                    }
                }
            }

            // if maxY is >= containing bounds max Y (off screen to the top)
            if (maxY >= waterBounds.getMax().Y)
            {
                // We want to make sure the Y velocity is negative.
                if (newVelocity.Y > 0)
				{
                    float min = waterBounds.getMax().Y - 10;
                    float max = waterBounds.getMax().Y;
                    float value = maxY / max;
                    float drag = N_Math.fitToRange(value, Normal_DragFactor, 0.0f);
                    newVelocity.Y *= drag;
				}
            }

            // set velocity again.
            this.setVelocity(newVelocity);
        }

        /// <summary>
        /// Keeps the hero from penetrating bounds.
        /// </summary>
        /// <param name="heroBounds">The hero bounds.</param>
        /// <param name="collisionPosition">The collision position.</param>
        private void KeepHeroFromPenetratingBounds(UWB_BoundingBox heroBounds, Vector3 collisionPosition)
        {
            Vector3 newVelocity = this.getVelocity();

            float minX = heroBounds.getMin().X;
            float maxX = heroBounds.getMax().X;
            float minY = heroBounds.getMin().Y;
            float maxY = heroBounds.getMax().Y;

            // if minX is <= collision position (intersecting on left of hero)
            if (minX <= collisionPosition.X)
            {
                // We want to make sure the X velocity is positive.
                if (newVelocity.X < 0)
                {
                    newVelocity.X *= -1; // if negative, reverse it.
                }
            }

            // if maxX is >= collision position (intersecting on right of hero)
            if (maxX >= collisionPosition.X)
            {
                // We want to make sure the X velocity is negative.
                if (newVelocity.X > 0)
                {
                    newVelocity.X *= -1; // if negative, reverse it.
                }
            }

            // if minY is <= collision position (intersecting below hero)
            if (minY <= collisionPosition.Y)
            {
                // We want to make sure the Y velocity is positive.
                if (newVelocity.Y < 0)
                {
                    newVelocity.Y *= -1; // if negative, reverse it.
                }
            }

            // if maxY is >= collision position (intersecting above hero)
            if (maxY >= collisionPosition.Y)
            {
                // We want to make sure the Y velocity is negative.
                if (newVelocity.Y > 0)
                {
                    newVelocity.Y *= -1; // if negative, reverse it.
                }
            }

            // set velocity again.
            this.setVelocity(newVelocity);
        }

        protected override Animation getAnimation(eAnimations animation)
        {
            switch (animation)
            {
                case eAnimations.Death: return createHitAnimation();
                case eAnimations.Idle: return createSwimAnimation();
                case eAnimations.SawShark: return createSawSharkAnimation();
            }

            return base.getAnimation(animation);
        }

        public void SawShark()
        {
            //Animate Scuba Dude
            playAnimation(eAnimations.SawShark);
        }

        private Animation createSwimAnimation()
        {
            AnimationList animList = new AnimationList();

            //left leg kick one way
            UWB_SceneNode node = findNode("Left Leg");
            AnimationQue que = new AnimationQue();
            if (node != null)
            {
                //get the current transform of the head
                UWB_XFormInfo targetXform = new UWB_XFormInfo(node.getXFormInfo());

                //Change the rotation
                targetXform.setRotationDegrees(50.0f);

                //Push it onto our stack
                que.add(new AnimationInfo(0.2f, node, targetXform));

                //Change the rotation
                targetXform.setRotationDegrees(-50.0f);

                //Push it onto our stack so it happens after the last
                que.add(new AnimationInfo(0.2f, node, targetXform));

                //Change the rotation
                targetXform.setRotationDegrees(0.0f);

                //Push it onto our stack so it happens after the last
                que.add(new AnimationInfo(0.2f, node, targetXform));
            }
            animList.add(que);

            //right leg kick the other way
            node = findNode("Right Leg");
            que = new AnimationQue();
            if (node != null)
            {
                //get the current transform of the head
                UWB_XFormInfo targetXform = new UWB_XFormInfo(node.getXFormInfo());

                //Change the rotation
                targetXform.setRotationDegrees(-50.0f);

                //Push it onto our stack
                que.add(new AnimationInfo(0.2f, node, targetXform));

                //Change the rotation
                targetXform.setRotationDegrees(50.0f);

                //Push it onto our stack so it happens after the last
                que.add(new AnimationInfo(0.2f, node, targetXform));

                //Change the rotation
                targetXform.setRotationDegrees(0.0f);

                //Push it onto our stack so it happens after the last
                que.add(new AnimationInfo(0.2f, node, targetXform));
            }
            animList.add(que);

            return new Animation(animList, true);
        }

        /// <summary>
        /// Creates the hit animation.
        /// </summary>
        private Animation createHitAnimation()
        {
            AnimationQue mainQue = new AnimationQue();

            //First make the eye way bigger then smaller, then normal, then so tiny you can't see it.
            UWB_SceneNode node = findNode("Eye");
            AnimationQue que = new AnimationQue();
            if (node != null)
            {
                //get the current transform of the head
                UWB_XFormInfo targetXform = new UWB_XFormInfo(node.getXFormInfo());

                //Change the scale
                targetXform.setScale(new Vector3(4.5f, 4.5f, 0.0f));

                //Push it onto our stack
                que.add(new AnimationInfo(0.2f, node, targetXform));

                //Change the scale
                targetXform.setScale(new Vector3(0.5f, 0.5f, 0.0f));

                //Push it onto our stack so it happens after the last
                que.add(new AnimationInfo(0.2f, node, targetXform));

                //Change the scale
                targetXform.setScale(node.getXFormInfo().getScale());

                //Push it onto our stack so it happens after the last
                que.add(new AnimationInfo(0.2f, node, targetXform));

                //Change the scale
                targetXform.setScale(new Vector3(.01f, .01f, 0.0f));

                //Push it onto our stack so it happens after the last
                que.add(new AnimationInfo(0.2f, node, targetXform));
            }
            mainQue.add(que);

            //After that make the head rotate while kicking the feet
            AnimationList animList = new AnimationList();
            mainQue.add(animList);

            //Head rotate
            node = findNode("Head");
            que = new AnimationQue();
            if (node != null)
            {
                //get the current transform of the head
                UWB_XFormInfo targetXform = new UWB_XFormInfo(node.getXFormInfo());

                //Change the rotation
                targetXform.setRotationDegrees(50.0f);

                //Push it onto our stack
                que.add(new AnimationInfo(0.2f, node, targetXform));

                //Change the rotation
                targetXform.setRotationDegrees(-50.0f);

                //Push it onto our stack so it happens after the last
                que.add(new AnimationInfo(0.2f, node, targetXform));

                //Change the rotation
                targetXform.setRotationDegrees(node.getXFormInfo().getRotationInDegrees());

                //Push it onto our stack so it happens after the last
                que.add(new AnimationInfo(0.2f, node, targetXform));
            }
            animList.add(que);

            //left leg kick one way
            node = findNode("Left Leg");
            que = new AnimationQue();
            if (node != null)
            {
                //get the current transform of the head
                UWB_XFormInfo targetXform = new UWB_XFormInfo(node.getXFormInfo());

                //Change the rotation
                targetXform.setRotationDegrees(50.0f);

                //Push it onto our stack
                que.add(new AnimationInfo(0.2f, node, targetXform));

                //Change the rotation
                targetXform.setRotationDegrees(-50.0f);

                //Push it onto our stack so it happens after the last
                que.add(new AnimationInfo(0.2f, node, targetXform));

                //Change the rotation
                targetXform.setRotationDegrees(node.getXFormInfo().getRotationInDegrees());

                //Push it onto our stack so it happens after the last
                que.add(new AnimationInfo(0.2f, node, targetXform));
            }
            animList.add(que);

            //right leg kick the other way
            node = findNode("Right Leg");
            que = new AnimationQue();
            if (node != null)
            {
                //get the current transform of the head
                UWB_XFormInfo targetXform = new UWB_XFormInfo(node.getXFormInfo());

                //Change the rotation
                targetXform.setRotationDegrees(-50.0f);

                //Push it onto our stack
                que.add(new AnimationInfo(0.2f, node, targetXform));

                //Change the rotation
                targetXform.setRotationDegrees(50.0f);

                //Push it onto our stack so it happens after the last
                que.add(new AnimationInfo(0.2f, node, targetXform));

                //Change the rotation
                targetXform.setRotationDegrees(node.getXFormInfo().getRotationInDegrees());

                //Push it onto our stack so it happens after the last
                que.add(new AnimationInfo(0.2f, node, targetXform));
            }
            animList.add(que);

            return new Animation(mainQue);
        }

        public Animation createSawSharkAnimation()
        {
            AnimationQue mainQue = new AnimationQue();

            //First make the eye bigger then smaller, then normal
            UWB_SceneNode node = findNode("Eye");
            AnimationQue que = new AnimationQue();
            if (node != null)
            {
                //get the current transform of the head
                UWB_XFormInfo targetXform = new UWB_XFormInfo(node.getXFormInfo());

                //Change the scale
                targetXform.setScale(new Vector3(1.5f, 1.5f, 0.0f));

                //Push it onto our stack
                que.add(new AnimationInfo(0.2f, node, targetXform));

                //Change the scale
                targetXform.setScale(new Vector3(0.5f, 0.5f, 0.0f));

                //Push it onto our stack so it happens after the last
                que.add(new AnimationInfo(0.2f, node, targetXform));

                //Change the scale
                targetXform.setScale(node.getXFormInfo().getScale());

                //Push it onto our stack so it happens after the last
                que.add(new AnimationInfo(0.2f, node, targetXform));
            }
            mainQue.add(que);

            return new Animation(mainQue);
        }

        public void aimAt(UWB_DrawHelper helper, Vector3 aimLocation)
        {
            if (!IsAlive)
                return;

            UWB_XFormInfo xform = new UWB_XFormInfo(getXFormInfo());

            // get X scale, because if he is facing left, we need to subtract instead of add.
            float xScale = xform.getScale().X;
            bool add = xScale > 0 ? true : false;

            float heroRotationDegrees = xform.getRotationInDegrees();

            UWB_SceneNode torso = findNode("Torso", true, true);
            if (torso != null)
            {
                AddOrSubtractRotation(xform, torso.getXFormInfo(), add);
            }

            UWB_SceneNode rightArm = torso.findNode("Right Arm", true, true);
            if (rightArm != null)
            {
                AddOrSubtractRotation(xform, rightArm.getXFormInfo(), add);

                Vector3 pivot = xform.getPivot();

                // transform with current xForm to get pivot world position
                helper.pushModelTransform();
                {
                    xform.setupModelStack(helper);

                    helper.transformPoint(ref pivot);
                }
                helper.popModelTransform();

                //get the angle from his shoulder to the aim location 
                float angle = (float)N_Math.getAngleBetweenPoints(pivot, aimLocation);

                angle = 180 - angle - heroRotationDegrees;

                //set the arms rotation to face the aim location
                rightArm.getXFormInfo().setRotationDegrees(add ? angle : -angle);
            }
        }
    }
}
