﻿using Microsoft.Xna.Framework;
using UWBGL_XNA_Lib13;
using System.Collections.Generic;
using NathanScott_Lib;
using System.Runtime.Serialization;
using System;
using System.Xml.Serialization;

namespace FinalProject
{
    /// <summary>
    /// This Class has a position, rotation, and scale that can easily be gotten and set. It contains the base for animations as well.
    /// </summary>
    [Serializable]
    [XmlInclude(typeof(Wall))]
    [XmlInclude(typeof(Boat))]
    [XmlInclude(typeof(Shark))]
    [XmlInclude(typeof(Spear))]
    public class SceneObject : UWB_SceneNode
    {
        Dictionary<eAnimations, Animation> mAnimations = new Dictionary<eAnimations, Animation>();
        protected eSceneObjectType mObjType = eSceneObjectType.Safe; // default

        public SceneObject()
            : base()
        {
        }

        public SceneObject(string name)
            : base(name)
        {
        }

        public SceneObject(Vector3 at, string name):base(name)
        {
            mXFormInfo.setPivot(at);
        }

        /// <summary>
        /// Response to a collision with another SceneObject.
        /// </summary>
        /// <param name="other">The other SceneObject this one collided with.</param>
        /// <param name="collisionPosition">The collision position.</param>
        public virtual void CollisionResponse(SceneObject other, Vector3 collisionPosition) { }

        public void playAnimation(eAnimations animation)
        {
            //Check if we already added the animation and if so play it
            Animation anim = null;
            if (mAnimations.TryGetValue(animation, out anim))
            {
                anim.play();
            }
            else
            {
                //If we haven't added it do so
                anim = getAnimation(animation);
                if (anim != null)
                    mAnimations.Add(animation, anim);
            }
        }

        /// <summary>
        /// Gets the type of the scene object.
        /// </summary>
        /// <returns></returns>
        public eSceneObjectType getSceneObjectType()
        {
            return this.mObjType;
        }

        public bool stopAnimation(eAnimations animation)
        {
            return mAnimations.Remove(animation);
        }

        public bool pauseAnimation(eAnimations animation)
        {
            //If we have the animation pause it
            Animation anim = null;
            if (mAnimations.TryGetValue(animation, out anim))
            {
                anim.pause();
                return true;
            }

            return false;
        }

        public bool finishAnimation(eAnimations animation)
        {
            //If we have the animation pause it
            Animation anim = null;
            if (mAnimations.TryGetValue(animation, out anim))
            {
                anim.stopLooping();
                return true;
            }

            return false;
        }

        public void animate(float elapsedSeconds)
        {
            //List to keep track of finished animations
            List<eAnimations> list = new List<eAnimations>();

            //Loop through all our animations and update them or
            //add them to the list if they are finished
            foreach (KeyValuePair<eAnimations, Animation> pair in mAnimations)
            {
                if (pair.Value.isFinished())
                {
                    list.Add(pair.Key);
                }
                else
                    pair.Value.update(elapsedSeconds);
            }

            //Handle any finished animations
            for (int i = 0; i < list.Count; i++)
            {
                eAnimations key = list[i];
                Animation value = null;
                if (mAnimations.TryGetValue(key, out value))
                {
                    bool isLooping = value.isLooping();

                    //Remove the item that is finished
                    mAnimations.Remove(key);

                    //If we are looping added it back in again (may be a better way of doing this)
                    if (isLooping)
                    {
                        Animation anim  = getAnimation(key);
                        if(anim != null)
                            mAnimations.Add(key, getAnimation(key));
                    }
                }
            }
        }

        /// <summary>
        /// Check our speed and if it is close or equal to zero we aren't moving
        /// </summary>
        /// <returns>true if moving, false if not</returns>
        public virtual bool isMoving()
        {
            //If we are basically stopped
            float speed = mVelocity.Length();
            return !((speed >= 0.0f && speed < 0.2f) || (speed <= 0.0f && speed > -0.2f));
        }

        public override void moveNodeByVelocity(float elapsedSecond)
        {
            base.moveNodeByVelocity(elapsedSecond);

            if (isMoving())
            {
                playAnimation(eAnimations.Idle);
            }
            else
            {
                pauseAnimation(eAnimations.Idle);
            }
        }

        protected void rotateWithVelocity()
        {
            float angle = N_Math.getAngleFromVelocity(mVelocity);
            if (angle > 90 || angle < -90)
            {
                angle = -angle + 180;
                Scale = new Vector3(-1, 1, 1);
            }
            else if (angle != 0.0f || mVelocity.X > 0)
            {
                Scale = new Vector3(1, 1, 1);
            }

            Rotation = angle;
        }

        protected virtual Animation getAnimation(eAnimations animation)
        {
            return null;
        }

        /// <summary>
        /// Gets/Sets the position of the scene object
        /// </summary>
        public Vector3 Position
        {
            get
            {
                return mXFormInfo.getTranslation();
            }

            set
            {
                mXFormInfo.setTranslation(value);
            }
        }

        /// <summary>
        /// Gets/Sets the rotation (Degrees) of the scene object
        /// </summary>
        public float Rotation
        {
            get
            {
                return mXFormInfo.getRotationInDegrees();
            }

            set
            {
                mXFormInfo.setRotationDegrees(value);
            }
        }

        /// <summary>
        /// Gets/Sets the position of the scene object
        /// </summary>
        public Vector3 Scale
        {
            get
            {
                return mXFormInfo.getScale();
            }

            set
            {
                mXFormInfo.setScale(value);
            }
        }

        public Vector3 Velocity
        {
            get
            {
                return getVelocity();
            }
            set
            {
                setVelocity(value);
            }
        }
    }
}
