﻿using Microsoft.Xna.Framework;
using UWBGL_XNA_Lib13;
using System;
using System.Xml.Serialization;

namespace FinalProject
{
    [Serializable]
    public class Wall : SceneObject
    {
        // members - walls need to build on each other, so they have these to quickly tell other walls about themselves.
        private float mLength = 0;
        private Vector3 mMin = Vector3.Zero;
        private float mThickness = 0;
        private eWallTextures mWallTexture;
        private eOrientation mOrientation;

        public Wall()
            : base()
        {

        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Wall"/> class.
        /// </summary>
        /// <param name="min">The min.</param>
        /// <param name="length">The length.</param>
        /// <param name="wallTexture">The wall texture.</param>
        /// <param name="orientation">The orientation.</param>
        public Wall(Vector3 min, float length, eWallTextures wallTexture, eOrientation orientation)
            : this(min, length, 15, new Color(139, 69, 19), wallTexture, orientation) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="Wall"/> class.
        /// </summary>
        /// <param name="min">The min.</param>
        /// <param name="length">The length.</param>
        /// <param name="thickness">The thickness.</param>
        /// <param name="color">The color.</param>
        /// <param name="wallTexture">The wall texture.</param>
        /// <param name="orientation">The orientation.</param>
        public Wall(Vector3 min, float length, float thickness, Color color, eWallTextures wallTexture, eOrientation orientation)
            : base("Wall")
        {
            this.mLength = length;
            this.mMin = min;
            this.mThickness = thickness;
            this.mWallTexture = wallTexture;
            mOrientation = orientation;

            mObjType = eSceneObjectType.Safe; // no harm here.

            UWB_PrimitiveList pList = new UWB_PrimitiveList();

            // build and append our rectangle
            pList.append(buildRectangleFromThisInstance(wallTexture, orientation, color));

            // default our velocity to zero.  We are a WALL.
            setVelocity(Vector3.Zero);

            setPrimitive(pList);
            mXFormInfo.setPivot(getCenter(orientation));
        }

        #region Public Properties for XML Serialization

        [XmlElement("Length")]
        public float LengthXML
        {
            get
            {
                return this.mLength;
            }
            set
            {
                this.mLength = value;
            }
        }

        [XmlElement("Thickness")]
        public float ThicknessXML
        {
            get
            {
                return this.mThickness;
            }
            set
            {
                this.mThickness = value;
            }
        }

        [XmlElement("MinPoint")]
        public Vector3 MinXML
        {
            get
            {
                return this.mMin;
            }
            set
            {
                this.mMin = value;
            }
        }

        [XmlElement("WallTexture")]
        public eWallTextures WallTextureXML
        {
            get
            {
                return this.mWallTexture;
            }
            set
            {
                this.mWallTexture = value;
            }
        }

        [XmlElement("Orientation")]
        public eOrientation Orientation
        {
            get
            {
                return mOrientation;
            }
            set
            {
                mOrientation = value;
            }
        }

        #endregion

        /// <summary>
        /// Returns the length of this wall
        /// </summary>
        /// <returns></returns>
        public float length()
        {
            return this.mLength;
        }

        /// <summary>
        /// Gets the min vector of this wall.
        /// </summary>
        /// <returns></returns>
        public Vector3 getMin()
        {
            return this.mMin;
        }

        /// <summary>
        /// Gets the thickness.
        /// </summary>
        /// <returns></returns>
        public float getThickness()
        {
            return this.mThickness;
        }

        /// <summary>
        /// Response to a collision with another SceneObject.
        /// </summary>
        /// <param name="other">The other SceneObject this one collided with.</param>
        /// <param name="collisionPosition">The collision position.</param>
        public override void CollisionResponse(SceneObject other, Vector3 collisionPosition)
        {
            base.CollisionResponse(other, collisionPosition);

            switch (other.getSceneObjectType())
            {
                case eSceneObjectType.Hero:
                    // play sound effect. too annoying.. took it out.
                    //Program.getModel().getSoundManager().PlaySound(SoundEffects.Thud, 1, 0, 0);

                    break;
                case eSceneObjectType.Deadly:
                case eSceneObjectType.Safe:
                    // do nothing...we don't care...
                default:
                    break;
            }
        }

        /// <summary>
        /// Builds the rectangle from this instance.
        /// </summary>
        /// <param name="wallTexture">The wall texture.</param>
        /// <param name="orientation">The orientation.</param>
        /// <param name="color">The color.</param>
        /// <returns></returns>
        private UWB_PrimitiveRectangle buildRectangleFromThisInstance(eWallTextures wallTexture, eOrientation orientation, Color color)
        {
            UWB_PrimitiveRectangle output = new UWB_PrimitiveRectangle();

            Vector3 max = getMaxPoint(orientation);
            output.setCorners(this.mMin, max);
            output.setFlatColor(color);
            output.setShadeMode(eShadeMode.smFlat);

            switch (wallTexture)
            {
                case eWallTextures.MetalBeam:
                    output.setTextureFileName("MetalBeam");
                    output.setTextureScale(8.45f);
                    break;
                case eWallTextures.Rock:
                    output.setTextureFileName("Rock");
                    output.setTextureScale(1.4f);
                    break;
                case eWallTextures.Sand:
                    output.setTextureFileName("Sand");
                    output.setTextureScale(10);
                    break;
                case eWallTextures.Wood:
                    // orientation matters for wood, so change it depending...
                    switch (orientation)
                    {
                        case eOrientation.Horizontal:
                            output.setTextureFileName("Wood4_horizontal");
                            if (mThickness > 15)
                            {
                                output.setTextureScale(1);
                            }
                            else
                            {
                                output.setTextureScale(1.6f);
                            }
                            break;
                        case eOrientation.Vertical:
                            output.setTextureFileName("Wood4_vertical");
                            if (mThickness > 15)
                            {
                                output.setTextureScale(1);
                            }
                            else
                            {
                                output.setTextureScale(1.6f);
                            }
                            break;
                        default:
                            break;
                    }
                    break;
                default:
                    break;
            }

            output.enableTexturing(true);
            output.useTextureAspectRatio(true);

            return output;
        }

        /// <summary>
        /// Gets the max point.
        /// </summary>
        /// <param name="orientation">The orientation.</param>
        /// <returns></returns>
        private Vector3 getMaxPoint(eOrientation orientation)
        {
            Vector3 max = Vector3.Zero;

            switch (orientation)
            {
                case eOrientation.Horizontal:
                    max = new Vector3(this.mMin.X + this.mLength, this.mMin.Y + this.mThickness, 0);
                    break;
                case eOrientation.Vertical:
                    max = new Vector3(this.mMin.X + this.mThickness, this.mMin.Y + this.mLength, 0);
                    break;
                default:
                    break;
            }

            return max;
        }

        /// <summary>
        /// Gets the center.
        /// </summary>
        /// <param name="orientation">The orientation.</param>
        /// <returns></returns>
        private Vector3 getCenter(eOrientation orientation)
        {
            Vector3 center = Vector3.Zero;

            switch (orientation)
            {
                case eOrientation.Horizontal:
                    center = new Vector3(this.mMin.X + (this.mLength / 2), this.mMin.Y + (this.mThickness / 2), 0);
                    break;
                case eOrientation.Vertical:
                    center = new Vector3(this.mMin.X + (this.mThickness / 2), this.mMin.Y + (this.mLength / 2), 0);
                    break;
                default:
                    break;
            }

            return center;
        }
    }
}
