﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using UWBGL_WinForms_Lib1;
using UWBGL_XNA_Lib13;
using NathanScott_Lib;
using System.Xml.Serialization;

namespace FinalProject
{
    public class TwoViewModel
    {
        protected const float MIN_ZOOM_SCALE = 0.1f;
        protected const float MAX_ZOOM_SCALE = 2f;

        #region Members

        /// <summary>
        /// Draw helper
        /// </summary>
        protected UWB_XNADrawHelper mDrawHelper;

        /// <summary>
        /// Text to hold what GUI we are currently in
        /// </summary>
        protected String mCurrentlyInText;
        /// <summary>
        /// Holds our the current hardware position of our mouse
        /// </summary>
        protected Vector2 mMouseHardwarePos;
        /// <summary>
        /// Holds the current device position of our mouse
        /// </summary>
        protected Vector3 mMouseDevicePos;
        /// <summary>
        /// Holds the current world position of our mouse
        /// </summary>
        protected Vector3 mMouseWorldPos;

        /// <summary>
        /// Original dimensions of the main view or world
        /// </summary>
        UWB_BoundingBox mOriginalWorldBounds;
        /// <summary>
        /// Current dimensions of the main view or world
        /// </summary>
        protected UWB_BoundingBox mWorldBounds;

        /// <summary>
        /// Original dimensions of the zoom in box (small view/zoomed in view)
        /// </summary>
        UWB_BoundingBox mOriginalZoomInBox;
        /// <summary>
        /// Current dimensions of the zoom in box (small view/zoomed in view)
        /// </summary>
        protected UWB_BoundingBox mZoomInBox;
        /// <summary>
        /// Current Zoom value (between MIN_ZOOM_SCALE and MAX_ZOOM_SCALE)
        /// i.e. 1 is the starting size, 2 is 2x the size...
        /// This is changed based on the right mouse button drag
        /// </summary>
        protected float mZoomInBoxZoom = 1.0f;
        /// <summary>
        /// After zooming is finished it is accumulated so you can continue to zoom out or in by clicking the mouse again.
        /// </summary>
        protected float mZoomInBoxZoomAccumulative = 1.0f;

        protected UWB_SceneNode mScene = null;

        /// <summary>
        /// Useful for calculating velocity, gravity, etc.
        /// </summary>
        protected UWB_Clock mClock;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="TwoViewModel"/> class.
        /// </summary>
        public TwoViewModel()
        {
            mDrawHelper = new UWB_XNADrawHelper();

            //Set up the world
            mWorldBounds = new UWB_BoundingBox(new Vector3(-90, -90, 0), new Vector3(90, 90, 0));
            mOriginalWorldBounds = new UWB_BoundingBox(mWorldBounds.getMin(), mWorldBounds.getMax());

            //Set up the zoom box/camera
            float origZoomInBoxHeight = 40;
            float origZoomInBoxWidth = 40;

            mZoomInBox = new UWB_BoundingBox(new Vector3(0, 0, 0), origZoomInBoxWidth, origZoomInBoxHeight, 0);
            mOriginalZoomInBox = new UWB_BoundingBox(mZoomInBox.getMin(), mZoomInBox.getMax());

            mZoomInBoxZoom = 1;
            mZoomInBoxZoomAccumulative = 1;

            mClock = new UWB_Clock();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TwoViewModel"/> class.
        /// </summary>
        /// <param name="worldWidth">Width of the world.</param>
        /// <param name="worldHeight">Height of the world.</param>
        /// <param name="cameraWidth">Width of the camera.</param>
        /// <param name="cameraHeight">Height of the camera.</param>
        public TwoViewModel(float worldWidth, float worldHeight, float cameraWidth, float cameraHeight)
        {
            mDrawHelper = new UWB_XNADrawHelper();

            //Set up the world
            mWorldBounds = new UWB_BoundingBox(new Vector3(0, 0, 0), worldWidth, worldHeight, 0);
            mOriginalWorldBounds = new UWB_BoundingBox(mWorldBounds.getMin(), mWorldBounds.getMax());

            //Set up the zoom box/camera
            mZoomInBox = new UWB_BoundingBox(new Vector3(0, 0, 0), cameraWidth, cameraHeight, 0);
            mOriginalZoomInBox = new UWB_BoundingBox(mZoomInBox.getMin(), mZoomInBox.getMax());

            mZoomInBoxZoom = 1;
            mZoomInBoxZoomAccumulative = 1;

            mClock = new UWB_Clock();
        }

        #endregion

        #region Public Properties for XML

        [XmlElement("WorldBounds")]
        public UWB_BoundingBox WorldBoundsXML
        {
            get
            {
                return this.mWorldBounds;
            }
            set
            {
                this.mWorldBounds = value;
            }
        }

        [XmlElement("OriginalWorldBounds")]
        public UWB_BoundingBox OriginalWorldBoundsXML
        {
            get
            {
                return this.mOriginalWorldBounds;
            }
            set
            {
                this.mOriginalWorldBounds = value;
            }
        }

        [XmlElement("ZoomInBox")]
        public UWB_BoundingBox ZoomInBoxXML
        {
            get
            {
                return this.mZoomInBox;
            }
            set
            {
                this.mZoomInBox = value;
            }
        }

        [XmlElement("OriginalZoomInBox")]
        public UWB_BoundingBox OriginalZoomInBoxXML
        {
            get
            {
                return this.mOriginalZoomInBox;
            }
            set
            {
                this.mOriginalZoomInBox = value;
            }
        }

        [XmlElement("ZoomInBoxZoom")]
        public float ZoomInBoxZoomXML
        {
            get
            {
                return this.mZoomInBoxZoom;
            }
            set
            {
                this.mZoomInBoxZoom = value;
            }
        }

        [XmlElement("ZoomInBoxZoomAccumulative")]
        public float ZoomInBoxZoomAccumulativeXML
        {
            get
            {
                return this.mZoomInBoxZoomAccumulative;
            }
            set
            {
                this.mZoomInBoxZoomAccumulative = value;
            }
        }

        #endregion

        #region Public Methods

        public void pause()
        {
            mClock.pause();
        }

        public void play()
        {
            mClock.play();
        }

        public void setSceneRoot(UWB_SceneNode root)
        {
            mScene = root;
        }

        public UWB_SceneNode getSceneRoot()
        {
            return mScene;
        }

        /// <summary>
        /// Updates the simulation.
        /// </summary>
        public virtual void updateSimulation()
        {
        }

        /// <summary>
        /// Gets the world bounds.
        /// </summary>
        /// <param name="width">The width.</param>
        /// <param name="height">The height.</param>
        public void getWorldBounds(ref float width, ref float height)
        {
            width = mWorldBounds.getWidth();
            height = mWorldBounds.getHeight();
        }

        /// <summary>
        /// Gets the world bounds.
        /// </summary>
        /// <returns>world bounds as a bounding box</returns>
        public UWB_BoundingBox getWorldBounds()
        {
            return mWorldBounds;
        }

        /// <summary>
        /// Draws the model.
        /// </summary>
        public virtual void drawModel()
        {
            if (mScene == null)
                return;

            eLevelofDetail lod = mDrawHelper.getLod();

            mDrawHelper.initializeModelTransform();
            mDrawHelper.pushModelTransform();

            // draw scene
            mScene.draw(lod, mDrawHelper);

            // pop our custom settings off
            mDrawHelper.popModelTransform();
        }

        /// <summary>
        /// Initialize the zoom view dimensions (should only be called once in the start)
        /// </summary>
        /// <param name="width">width of the zoom view window</param>
        /// <param name="height">height of the zoom view window</param>
        public void initZoomViewDimensions(float width, float height)
        {
            mZoomInBox.setWidth(width);
            mZoomInBox.setHeight(height);
            mOriginalZoomInBox = new UWB_BoundingBox(mZoomInBox.getMin(), mZoomInBox.getMax());
        }

        /// <summary>
        /// Set the center of the zoomed in view box
        /// </summary>
        /// <param name="center">center position for the zoomed in view in World coordinates</param>
        public void setZoomedViewCenter(Vector3 center)
        {
            mZoomInBox.setCenter(center);
        }

        /// <summary>
        /// Change the amount the zoomed view is zoomed in or out based on a distance and direction the mouse has moved
        /// </summary>
        /// <param name="distance">distance and direction mouse has moved in the x axis (positive vs negative)</param>
        public void setZoomedViewZoom(float distance)
        {
            //figure out how much to scale based on our mouse movement
            mZoomInBoxZoom = (float)N_Math.scaleToRange(distance, -45, 50, MIN_ZOOM_SCALE, MAX_ZOOM_SCALE);

            //Update the zoomed view dimensions based on the new zoom
            updateZoomInBoxDimensions();
        }

        /// <summary>
        /// Finished zooming so accumulate the zoom
        /// </summary>
        public void doneWithZoom()
        {
            mZoomInBoxZoomAccumulative *= mZoomInBoxZoom;
            mZoomInBoxZoom = 1;
        }

        /// <summary>
        /// Reset the zoomed view center, width, and height
        /// </summary>
        public void resetZoomedView()
        {
            mZoomInBoxZoomAccumulative = 1;
            mZoomInBoxZoom = 1;
            mZoomInBox.setCorners(mOriginalZoomInBox.getMin(), mOriginalZoomInBox.getMax());
            updateZoomInBoxDimensions();
        }

        /// <summary>
        /// Draw the zoomed view box as the a rectangle (red wireframe)
        /// </summary>
        public void drawZoomedViewBox()
        {
            eLevelofDetail lod = mDrawHelper.getLod();

            mDrawHelper.initializeModelTransform();
            mDrawHelper.pushModelTransform();

            mDrawHelper.setFillMode(eFillMode.fmWireframe);
            mDrawHelper.setColor1(Color.Red);
            mDrawHelper.setColor2(Color.Red);
            mDrawHelper.setShadeMode(eShadeMode.smFlat);
            mZoomInBox.draw(mDrawHelper);

            mDrawHelper.popModelTransform();
        }

        /// <summary>
        /// Sets the mouse info.
        /// </summary>
        /// <param name="currentlyInText">The currently in text.</param>
        /// <param name="hardwarePos">The hardware pos.</param>
        /// <param name="devicePos">The device pos.</param>
        /// <param name="worldPos">The world pos.</param>
        public void setMouseInfo(String currentlyInText, Vector2 hardwarePos, Vector3 devicePos, Vector3 worldPos)
        {
            mCurrentlyInText = currentlyInText;
            mMouseHardwarePos = hardwarePos;
            mMouseDevicePos = devicePos;
            mMouseWorldPos = worldPos;
        }

        /// <summary>
        /// Gets the currently in text.
        /// </summary>
        /// <returns></returns>
        public String getCurrintlyInText()
        {
            return mCurrentlyInText;
        }

        /// <summary>
        /// Gets the mouse hardware pos.
        /// </summary>
        /// <returns></returns>
        public Vector2 getMouseHardwarePos()
        {
            return mMouseHardwarePos;
        }

        /// <summary>
        /// Gets the mouse device pos.
        /// </summary>
        /// <returns></returns>
        public Vector3 getMouseDevicePos()
        {
            return mMouseDevicePos;
        }

        /// <summary>
        /// Gets the mouse world pos.
        /// </summary>
        /// <returns></returns>
        public Vector3 getMouseWorldPos()
        {
            return mMouseWorldPos;
        }

        /// <summary>
        /// Get the original bounds of the large view;
        /// </summary>
        /// <returns>original bounds of the large view</returns>
        public UWB_BoundingBox getOriginalWorldBounds()
        {
            return mOriginalWorldBounds;
        }

        /// <summary>
        /// Get a the dimensions of the zoom box
        /// </summary>
        /// <returns>dimensions of the zoom box</returns>
        public UWB_BoundingBox getZoomedInViewBox()
        {
            return mZoomInBox;
        }

        /// <summary>
        /// Get the original bounds of the zoom in box
        /// </summary>
        /// <returns></returns>
        public UWB_BoundingBox getOriginalZoomedInViewBox()
        {
            return mOriginalZoomInBox;
        }

        #endregion

        #region Event Service Methods



        #endregion

        #region Private Methods

        /// <summary>
        /// Update the width and height of the zoomed in view box based on our current zoom (mZoomInBoxZoom) and our width percentage
        /// </summary>
        private void updateZoomInBoxDimensions()
        {
            float width = mOriginalZoomInBox.getWidth();
            float height = mOriginalZoomInBox.getHeight();

            float size = Math.Min(width, height);
            height = size;

            //Apply the zoom (based on mouse drag)
            float newWidth = width * mZoomInBoxZoom * mZoomInBoxZoomAccumulative;

            //Calculate our new height
            float newHeight = N_Math.calculateHeight(width, height, newWidth);

            //Update the width and height
            mZoomInBox.setWidth(newWidth);
            mZoomInBox.setHeight(newHeight);
        }

        #endregion
    }
}

