﻿using System;
using System.Windows.Forms;
using Microsoft.Xna.Framework;
using UWBGL_XNA_Lib13;

namespace FinalProject
{
    public partial class ObjectEditorForm : Form
    {
        #region Members

        // viewports
        private ModelControlHandler mViewEdit;
        private CameraControlHandler mViewWorld;

        private UWB_PrimitiveList mCurPrimitiveList = null;

        private TwoViewModel mModel = new TwoViewModel(337, 400, 100, 100);

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ObjectEditorForm"/> class.
        /// </summary>
        public ObjectEditorForm()
        {
            InitializeComponent();

            mViewEdit = new ModelControlHandler(mModel);
            mViewWorld = new CameraControlHandler(mModel);
        }

        #endregion

        #region Public Methods

        public void setSceneRoot(UWB_SceneNode root)
        {
            mModel.setSceneRoot(root);
            refreshSceneNodeTreeView();

            if (root != null)
            {
                Vector3 pos = root.getXFormInfo().getPivot();
                pos += root.getXFormInfo().getTranslation();
                mModel.setZoomedViewCenter(pos);
            }
            else
                mModel.setZoomedViewCenter(Vector3.Zero);
        }

        public void play()
        {
            mTimer.Start();
        }

        public void pause()
        {
            mTimer.Stop();
        }

        #endregion

        #region Event Service Methods

        /// <summary>
        /// Handles the MouseMove event of the Mp4Form control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
        private void ObjectEditorForm_MouseMove(object sender, MouseEventArgs e)
        {
            System.Drawing.Point pos = this.PointToClient(Cursor.Position);
            mModel.setMouseInfo("Main Dialog", new Vector2(pos.X, pos.Y), Vector3.Zero, Vector3.Zero);
        }

        /// <summary>
        /// Handles the MouseLeave event of the Mp4Form control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ObjectEditorForm_MouseLeave(object sender, EventArgs e)
        {
            mModel.setMouseInfo("Unknown", Vector2.Zero, Vector3.Zero, Vector3.Zero);
        }

        /// <summary>
        /// Handles the Load event of the Mp4Form control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ObjectEditorForm_Load(object sender, EventArgs e)
        {
            mViewWorld.Initialize(mPlaceholderWorld);
            mViewEdit.Initialize(mPlaceholderHero);
            
            //Link our world bounds with our large viewport
            UWB_BoundingBox worldBoundLarge = mModel.getWorldBounds();
            mViewWorld.setWCWindow(ref worldBoundLarge);

            //Set the small zoom box to have the same height (entire head)
            //but change our width so that the aspect ratio stays the same as our small window
            float height = mModel.getZoomedInViewBox().getHeight();
            float width = mModel.getZoomedInViewBox().getWidth();
            float ratio = mViewEdit.getAspectRatio();
            width = height * ratio;
            mModel.initZoomViewDimensions(width, height);

            //Build our tree view based on our scene
            refreshSceneNodeTreeView();

            //Link our zoom view box to our small window
            UWB_BoundingBox worldBoundSmall = mModel.getZoomedInViewBox();
            mViewEdit.setWCWindow(ref worldBoundSmall);

            //Start our timer
            mTimer.Start();
        }

        /// <summary>
        /// Handles the Tick event of the mTimer control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void mTimer_Tick(object sender, EventArgs e)
        {            
            //Update the simulation (collision, movement...)
            mModel.updateSimulation();

            //Draw Viewports
            mViewWorld.DrawGraphics();
            mViewEdit.DrawGraphics();
        }

        /// <summary>
        /// Handles the AfterSelect event of the _tvHeroLimbSelect control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.TreeViewEventArgs"/> instance containing the event data.</param>
        private void _tvSceneNodeSelect_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if ((this._tvSceneNodeSelect != null) && (this._tvSceneNodeSelect.Nodes != null) &&
                (this._tvSceneNodeSelect.Nodes.Count > 0) && e.Node.Tag is UWB_SceneNode)
            {
                //get the scene node this corresponds to
                UWB_SceneNode sceneNode = e.Node.Tag as UWB_SceneNode;

                //Update our references
                updatePrimitveReference(sceneNode.getPrimitive());
                mXFormInfo.setSceneNode(sceneNode);
            }
        }

        private void _sldrNthPrimitive_TypeValidText(double value)
        {
            int index = Convert.ToInt32(value);
            if (mCurPrimitiveList != null && index >= 0 && index < mCurPrimitiveList.count())
            {
                UWB_Primitive primitive = mCurPrimitiveList.primitiveAt(index);

                mPrimitiveControl.setPrimitive(primitive);
            }
        }

        protected override void OnFormClosing(FormClosingEventArgs e)
        {
            Program.closeObjectEditor();
            e.Cancel = true;
        }

        #endregion

        /// <summary>
        /// When we select a new scene node we need to update everything with the new primitive
        /// </summary>
        /// <param name="primitive">new primitive to use for our menus</param>
        private void updatePrimitveReference(UWB_Primitive primitive)
        {
            if(primitive != null)
            {
                if (primitive is UWB_PrimitiveList)
                {
                    //Store a reference to the list we are using and the max of it.
                    mCurPrimitiveList = primitive as UWB_PrimitiveList;
                    _sldrNthPrimitive.Maximum = mCurPrimitiveList.count();
                }
                else
                {
                    //No list so null us out and set the max to 0
                    mCurPrimitiveList = null;
                    _sldrNthPrimitive.Maximum = 0;

                    //We just have one so set it
                    mPrimitiveControl.setPrimitive(primitive);
                }

                //Reset our index to 0 no matter what
                _sldrNthPrimitive.Value = 0.0;
                _sldrNthPrimitive_TypeValidText(_sldrNthPrimitive.Value);
            }
            else
                mPrimitiveControl.setPrimitive(null);
        }

        /// <summary>
        /// Clear the tree view and rebuild it based on our model's scene.
        /// Warning: Every time the models scene changes this should be called (references to scene nodes)
        /// </summary>
        private void refreshSceneNodeTreeView()
        {
            if (mModel == null)
                return;

            //Clear out whatever was there before
            _tvSceneNodeSelect.Nodes.Clear();

            //Get a reference to our scene
            UWB_SceneNode root = mModel.getSceneRoot();
            if (root == null)
                return;

            //Populate the Tree
            UWB_SceneNode sceneNode = root;
            TreeNode treeNode = new TreeNode(sceneNode.getName());
            treeNode.Tag = sceneNode;
            _tvSceneNodeSelect.Nodes.Add(treeNode);

            refreshSceneNodeTreeViewHelper(ref treeNode, sceneNode);
        }

        /// <summary>
        /// Helper function to recursively build the tree based on the current scene node we are on
        /// </summary>
        /// <param name="parentTreeNode">reference to tree's node we should add the children to</param>
        /// <param name="currentSceneNode">current scene node we are on to get the children off</param>
        private void refreshSceneNodeTreeViewHelper(ref TreeNode parentTreeNode, UWB_SceneNode currentSceneNode)
        {
            for (int i = 0; i < currentSceneNode.numChildren(); i++)
            {
                UWB_SceneNode sceneNode = currentSceneNode.getChildNode(i);

                //Set the name that will be displayed
                TreeNode treeNode = new TreeNode(sceneNode.getName());

                //Save a reference to our scene node
                treeNode.Tag = sceneNode;
                
                //Build all the children
                refreshSceneNodeTreeViewHelper(ref treeNode, sceneNode);

                //Add the node to the tree
                parentTreeNode.Nodes.Add(treeNode);
            }
        }
    }
}
