﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using NathanScott_Lib;
using FinalProject.Windows;
using UWBGL_XNA_Lib13;

namespace FinalProject
{
    public partial class StoreWindow : InventoryForm
    {
        //Callbacks
        public delegate void StoreCloseEventHanlder();

        public event StoreCloseEventHanlder StoreCloseCallback;

        private Inventory mStore = null;
        private Hero mPerson = null;

        public StoreWindow()
        {
            InitializeComponent();
            _tabControl.SelectedTab = _tpBuy;
        }

        public void ini(Inventory store, Hero person)
        {
            mStore = store;
            mPerson = person;

            //Populate the tables
            updateTable(_tblBuy, mStore, onBuyButtonCreated, false);
            updateTable(_tblSell, mPerson.Bag, onSellButtonCreated, false);
            updateTable(_tblRecover, mPerson.Inventory, onRecoverButtonCreated, true);

            updatePerformAllButton();
        }

        protected override bool itemCheck(Item item)
        {
            return item.isRecoveryable();
        }

        /// <summary>
        /// This is called each loop in the populate control so you can
        /// customize your button or whatever else you want to do each loop
        /// </summary>
        /// <param name="button">button to be customized</param>
        /// <param name="item">item to use to customize it with/it corresponds to</param>
        private void onBuyButtonCreated(Button button, Item item)
        {
            //set the button callback
            button.Click += new EventHandler(this.onBuyButtonClicked);

            //set a random price between our ranges
            Random ran = new Random();
            item.Price = (float)N_Math.fitToRange(ran.NextDouble(), item.MinPrice, item.MaxPrice);

            //Customize our button
            button.BackgroundImage = item.PreviewImage;

            button.BackgroundImageLayout = ImageLayout.Zoom;
            button.Text = item.Price.ToString("$0.00");
            button.TextAlign = ContentAlignment.BottomRight;
            mToolTip.SetToolTip(button, item.getName());
        }

        /// <summary>
        /// Called when a buy button is clicked
        /// </summary>
        /// <param name="sender">button instance that was clicked</param>
        /// <param name="e">other args</param>
        private void onBuyButtonClicked(object sender, EventArgs e)
        {
            if (sender is Button)
            {
                Button button = sender as Button;

                if (button.Tag is Item)
                {
                    Item item = button.Tag as Item;

                    //This handles checking the money and subtracting it
                    if (mPerson.addInventoryItem(item))
                    {
                        //Purchased it so remove it
                        mStore.removeItem(item);
                        _tblBuy.Controls.Remove(button);

                        // play cash register sound
                        playCashRegisterSound();

                        //Important to let the user know is current money and air
                        Program.updateUI();
                    }
                    else
                    {
                        string message = "You do not have enough money. The item costs ";
                        message += item.Price.ToString("$0.00") + " and you only have ";
                        message += mPerson.Money.ToString("$0.00");
                        MessageBox.Show(message);
                    }
                }
            }
        }

        private void playCashRegisterSound()
        {
            UWB_Manager.mAudioManager.playSound(eSoundFiles.CashRegister.ToString(), 1, 0, 0);
        }

        /// <summary>
        /// This is called each loop in the populate control so you can
        /// customize your button or whatever else you want to do each loop
        /// </summary>
        /// <param name="button">button to be customized</param>
        /// <param name="item">item to use to customize it with/it corresponds to</param>
        private void onSellButtonCreated(Button button, Item item)
        {
            //set the button callback
            button.Click += new EventHandler(this.onSellButtonClicked);

            Random ran = new Random();

            if (item is Collectible) // if it's a collectible, use the condition and level of the collectible to set prices for this store
            {
                item.Price = generatePriceForCollectible(((Collectible)(item)).Condition, ((Collectible)(item)).Level);
            }
            else if (item is Item) //if it's a normal item, set a random price between our ranges
            {
                item.Price = N_Utility.GenerateRandomFloat(item.MinPrice, item.MaxPrice);
            }

            //Customize our button
            button.BackgroundImage = item.PreviewImage;
            button.BackgroundImageLayout = ImageLayout.Stretch;
            button.Text = item.Price.ToString("$0.00");
            button.TextAlign = ContentAlignment.BottomRight;
            mToolTip.SetToolTip(button, item.getName());
        }

        /// <summary>
        /// Generates the price for collectible.
        /// </summary>
        /// <param name="condition">The condition.</param>
        /// <param name="level">The level.</param>
        /// <returns></returns>
        private float generatePriceForCollectible(eCollectibleCondition condition, int level)
        {
            float output = 0;

            // set base price using condition
            switch (condition)
            {
                case eCollectibleCondition.Mint:
                    output = N_Utility.GenerateRandomFloat(75, 115);
                    break;
                case eCollectibleCondition.Good:
                    output = N_Utility.GenerateRandomFloat(40, 75);
                    break;
                case eCollectibleCondition.Fair:
                    output = N_Utility.GenerateRandomFloat(20, 40);
                    break;
                case eCollectibleCondition.Poor:
                    output = N_Utility.GenerateRandomFloat(5, 20);
                    break;
                default:
                    break;
            }

            // now factor in level
            output *= level;

            return output;
        }

        /// <summary>
        /// Called when a sell button is clicked
        /// </summary>
        /// <param name="sender">button instance that was clicked</param>
        /// <param name="e">other args</param>
        private void onSellButtonClicked(object sender, EventArgs e)
        {
            if (sender is Button)
            {
                Button button = sender as Button;

                if (button.Tag is Item)
                {
                    Item item = button.Tag as Item;

                    //Handles adding the money and removing it
                    if (mPerson.sellCollectible(item))
                    {
                        //Sold it so remove it
                        _tblSell.Controls.Remove(button);

                        updatePerformAllButton();

                        // play cash register sound
                        playCashRegisterSound();

                        //Important to let the user know is current money and air
                        Program.updateUI();
                    }
                    else
                    {
                        string message = "For some reason you don't have this item?";
                        MessageBox.Show(message);
                    }
                }
            }
        }

        /// <summary>
        /// This is called each loop in the populate control so you can
        /// customize your button or whatever else you want to do each loop
        /// </summary>
        /// <param name="button">button to be customized</param>
        /// <param name="item">item to use to customize it with/it corresponds to</param>
        private void onRecoverButtonCreated(Button button, Item item)
        {
            //set the button callback
            button.Click += new EventHandler(this.onRecoverButtonClicked);

            //Customize our button
            button.BackgroundImage = item.mRecoverImage;
            button.BackgroundImageLayout = ImageLayout.Stretch;
            button.Text = item.getRecoveryText();
            button.TextAlign = ContentAlignment.BottomRight;
        }

        /// <summary>
        /// Called when a recover button is clicked
        /// </summary>
        /// <param name="sender">button instance that was clicked</param>
        /// <param name="e">other args</param>
        private void onRecoverButtonClicked(object sender, EventArgs e)
        {
            if (sender is Button)
            {
                Button button = sender as Button;

                if (button.Tag is Item)
                {
                    Item item = button.Tag as Item;

                    if (item.isRecoveryable())
                    {
                        //Do whatever we should do
                        if (item.performRecoverAction(mPerson))
                        {
                            //Action done so remove it if we can't do it again
                            if(!item.isRecoveryable())
                            {
                                _tblRecover.Controls.Remove(button);
                                updatePerformAllButton();
                            }

                            // play cash register sound
                            playCashRegisterSound();

                            //Important to let the user know is current money and air
                            Program.updateUI();
                        }
                        else
                        {
                            string message = "Don't have enough money.";
                            MessageBox.Show(message);
                        }
                    }
                    else
                    {
                        string message = "For some reason this doesn't have a recover action. Why is it in this menu?";
                        MessageBox.Show(message);
                    }
                }
            }
        }

        private void _btnDone_Click(object sender, EventArgs e)
        {
            if (this.StoreCloseCallback != null)
                this.StoreCloseCallback();

            Close();
        }

        private void _btnPerformAll_Click(object sender, EventArgs e)
        {
            TableLayoutPanel table = null;

            //Get the correct table
            if (_tabControl.SelectedTab == _tpSell)
            {
                table = _tblSell;
            }
            else if (_tabControl.SelectedTab == _tpRecover)
            {
                table = _tblRecover;
            }

            //Perform the action on all buttons in the table
            if (table != null)
            {
                for (int i = table.Controls.Count - 1; i >= 0; i--)
                {
                    Control ctrl = table.Controls[i];

                    if (ctrl is Button)
                    {
                        Button button = ctrl as Button;
                        button.PerformClick();
                    }
                }

                updatePerformAllButton();
            }
        }

        private void _tabControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            updatePerformAllButton();
        }

        private void updatePerformAllButton()
        {
            TabPage selectedTab = _tabControl.SelectedTab;
            if (selectedTab == _tpBuy)
            {
                _btnPerformAll.Visible = false;
            }
            else if (selectedTab == _tpSell)
            {
                _btnPerformAll.Visible = true;
                _btnPerformAll.Text = "Sell All";

                if (_tblSell.Controls.Count > 0)
                    _btnPerformAll.Enabled = true;
                else
                    _btnPerformAll.Enabled = false;
            }
            else if (selectedTab == _tpRecover)
            {
                _btnPerformAll.Visible = true;
                _btnPerformAll.Text = "Recover All";

                if (_tblRecover.Controls.Count > 0)
                    _btnPerformAll.Enabled = true;
                else
                    _btnPerformAll.Enabled = false;
            }
        }

        private void _tpBuy_Click(object sender, EventArgs e)
        {
        }

        private void _tpSell_Click(object sender, EventArgs e)
        {
        }

        private void _tpRecover_Click(object sender, EventArgs e)
        {
        }
    }
}
