﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace NathanScott_Lib
{
    public static class N_Math
    {
        public const float PI = 3.14159265358979323846f;
        public const float VERY_LARGE = 999999999.0f;
        public const float VERY_SMALL = -VERY_LARGE;
        public const float ZERO_TOLERANCE = (float)1.0e-9;

        /// <summary>
        /// Fit a value between a range
        /// </summary>
        /// <param name="value">Number between 0.0 and 1.0 to be fit into the new range</param>
        /// <param name="min">Minimum value of the range</param>
        /// <param name="max">Maximum value of the range</param>
        /// <returns>Number between the range</returns>
        /// <example>
        /// Random ran = new Random();
        /// double value = FitToRange(ran.NextDouble(), 0.0, 200.0);
        /// </example>
        public static double fitToRange(double value, double min, double max)
        {
            return (max - min) * value + min;
        }

        /// <summary>
        /// Fit a value between a range
        /// </summary>
        /// <param name="value">Number between 0.0 and 1.0 to be fit into the new range</param>
        /// <param name="min">Minimum value of the range</param>
        /// <param name="max">Maximum value of the range</param>
        /// <returns>Number between the range</returns>
        /// <example>
        /// Random ran = new Random();
        /// double value = FitToRange(ran.NextDouble(), 0.0, 200.0);
        /// </example>
        public static float fitToRange(float value, float min, float max)
        {
            return (max - min) * value + min;
        }

        /// <summary>
        /// Scale a value with a given range to a new range
        /// </summary>
        /// <param name="value">Value to be scaled</param>
        /// <param name="oldMin">old minimum (that value is within)</param>
        /// <param name="oldMax">old maximum (that value is within)</param>
        /// <param name="newMin">new minimum</param>
        /// <param name="newMax">new maximum</param>
        /// <returns>new scaled value, returns 0 if either old or new range is zero</returns>
        public static double scaleToRange(double value, double oldMin, double oldMax, double newMin, double newMax)
        {
            double oldRange = oldMax - oldMin;
            double newRange = newMax - newMin;
            value = N_Math.Clamp(value, oldMin, oldMax);
            value = value - oldMin;

            if (oldRange == 0.0D || newRange == 0.0D)
                return 0.0D;

            return N_Math.Clamp(value / oldRange * newRange + newMin, newMin, newMax);
        }

        /// <summary>
        /// Scale a value with a given range to a new range
        /// </summary>
        /// <param name="value">Value to be scaled</param>
        /// <param name="oldMin">old minimum (that value is within)</param>
        /// <param name="oldMax">old maximum (that value is within)</param>
        /// <param name="newMin">new minimum</param>
        /// <param name="newMax">new maximum</param>
        /// <returns>new scaled value, returns 0 if either old or new range is zero</returns>
        public static float scaleToRange(float value, float oldMin, float oldMax, float newMin, float newMax)
        {
            float oldRange = oldMax - oldMin;
            float newRange = newMax - newMin;
            value = N_Math.Clamp(value, oldMin, oldMax);
            value = value - oldMin;

            if (oldRange == 0.0D || newRange == 0.0D)
                return 0.0f;

            return N_Math.Clamp(value / oldRange * newRange + newMin, newMin, newMax);
        }

        /// <summary>
        /// Clamp a value between a range
        /// (if it is greater than the max set it to the max, if it is smaller than the min set it to the min)
        /// </summary>
        /// <param name="value">value to be clamped</param>
        /// <param name="min">minimum value for range</param>
        /// <param name="max">maximum value for range</param>
        /// <returns>new clamped value</returns>
        public static double Clamp(double value, double min, double max)
        {
            return Math.Max(Math.Min(value, max), min);
        }

        /// <summary>
        /// Clamp a value between a range
        /// (if it is greater than the max set it to the max, if it is smaller than the min set it to the min)
        /// </summary>
        /// <param name="value">value to be clamped</param>
        /// <param name="min">minimum value for range</param>
        /// <param name="max">maximum value for range</param>
        /// <returns>new clamped value</returns>
        public static float Clamp(float value, float min, float max)
        {
            return Math.Max(Math.Min(value, max), min);
        }

        /// <summary>
        /// Clamp a value between a range
        /// (if it is greater than the max set it to the max, if it is smaller than the min set it to the min)
        /// </summary>
        /// <param name="value">value to be clamped</param>
        /// <param name="min">minimum value for range</param>
        /// <param name="max">maximum value for range</param>
        /// <returns>new clamped value</returns>
        public static int Clamp(int value, int min, int max)
        {
            return Math.Max(Math.Min(value, max), min);
        }

        /// <summary>
        /// Calculate the width given the new height and original dimensions while maintaining aspect ratio
        /// </summary>
        /// <param name="origWidth">Original width</param>
        /// <param name="origHeight">Original height</param>
        /// <param name="height">new Height</param>
        /// <returns>Calculated width based on the new height</returns>
        public static float calculateWidth(float origWidth, float origHeight, float height)
        {
            float aspectRatio = origWidth / origHeight;
            float width = height * aspectRatio;
            return width;
        }

        /// <summary>
        /// Calculate the height given the new width and original dimensions while maintaining aspect ratio
        /// </summary>
        /// <param name="origWidth">Original width</param>
        /// <param name="origHeight">Original height</param>
        /// <param name="width">New width</param>
        /// <returns>Calculated height based on the new width</returns>
        public static float calculateHeight(float origWidth, float origHeight, float width)
        {
            float height = origHeight * width / origWidth;
            return height;
        }

        /// <summary>
        /// Converts a byte color value (0-255) to a double (0-1)
        /// </summary>
        /// <param name="value">color value (0-255)</param>
        /// <returns>color value (0-1)</returns>
        public static double colorByteToDouble(byte value)
        {
            return N_Math.Clamp(Convert.ToDouble(value) / 255.0, 0.0, 1.0);
        }

        /// <summary>
        /// Converts a byte color value (0-255) to a double (0-1)
        /// </summary>
        /// <param name="value">color value (0-255)</param>
        /// <returns>color value (0-1)</returns>
        public static float colorByteToFloat(byte value)
        {
            return N_Math.Clamp((float)value / 255.0f, 0.0f, 1.0f);
        }

        /// <summary>
        /// Converts a double color value (0-1) to a byte (0-255)
        /// </summary>
        /// <param name="value">color value (0-1)</param>
        /// <returns>color value (0-255)</returns>
        public static byte colorDoubleToByte(double value)
        {
            double d = N_Math.Clamp(value, 0.0f, 1.0f);
            return Convert.ToByte(Math.Floor(d == 1.0 ? 255.0D : d * 256.0D));
        }

        /// <summary>
        /// Converts a double color value (0-1) to a byte (0-255)
        /// </summary>
        /// <param name="value">color value (0-1)</param>
        /// <returns>color value (0-255)</returns>
        public static byte colorFloatToByte(float value)
        {
            float f = N_Math.Clamp(value, 0.0f, 1.0f);
            return Convert.ToByte(Math.Floor(f == 1.0f ? 255 : f * 256.0f));
        }

        public static double toRadians(double degree)
        {
            return ((degree) * (Math.PI / 180.0));
        }

        public static float toRadians(float degree)
        {
            return ((degree) * (PI / 180.0f));
        }

        public static double toDegrees(double radian)
        {
            return ((radian) * (180.0 / Math.PI));
        }        

        public static float toDegrees(float radian)
        {
            return ((radian) * (180.0f / PI));
        }

        public static Vector3 getPointOnLine(Vector3 start, Vector3 end, Vector3 point, float distance)
        {
            float totalDistance = Vector3.Distance(start, end);
            float k = distance / totalDistance;
            return (point + k * (start - end));
        }

        public static Vector3 getMidPoint(Vector3 start, Vector3 end)
        {
            return (start + end) / 2f;
        }

        public static Vector3 getDirection(Vector3 origion, Vector3 target)
        {
            Vector3 direction = target - origion;
            direction.Normalize();

            return direction;
        }

        public static Vector3 getVelocity(Vector3 origion, Vector3 target, float speed)
        {
            Vector3 direction = getDirection(origion, target);

            Vector3 velocity = direction * speed;

            return velocity;
        }

        public static float getAngleFromVelocity(Vector3 velocity)
        {
            if (velocity.X == 0.0f && velocity.Y == 0.0f)
                return 0.0f;
            else if (velocity.X >= 0.0f)
                return toDegrees((float)Math.Atan(velocity.Y / velocity.X));
            else if (velocity.X < 0.0f)
            {
                return toDegrees((float)(-Math.Atan(velocity.Y / velocity.X) + PI));
            }

            return 0.0f;
        }

        public static double getAngleBetweenPoints(Vector3 p1, Vector3 p2)
        {
            double angle = toDegrees(Math.Atan2(p2.X - p1.X, p2.Y - p1.Y));

            if (angle > 360)
                angle -= 360;
            if (angle < 0)
                angle += 360;

            return angle;
        }

        public static int round(double value)
        {
            return Convert.ToInt32(Math.Round(value));
        }
    }
}
