﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using NathanScott_Lib;

namespace SliderWithTextBox
{
    [ToolboxBitmapAttribute(typeof(SliderWithTextBoxControl),"SliderWithTextBox.Icon.bmp")]
    public partial class SliderWithTextBoxControl : UserControl
    {
        /// <summary>
        /// Value used to convert from int to float for the TrackBar values
        /// </summary>
        public const int SLIDER_RESOLUTION = 100000;

        private double mSliderBarRange;
        private double mMaximum;
        private double mMinimum;
        private double mLargeChange;
        private double mSmallChange;
        private double mTickFrequency;
        private double mValue;
        private bool mDisplayAsFloat;

        /// <summary>
        /// Constructor for our control
        /// </summary>
        public SliderWithTextBoxControl()
        {
            InitializeComponent();

            mDisplayAsFloat = true;

            updateSliderBarRange();
            updateSliderBar();
            updateTextBox();
        }

        /// <summary>
        /// Set the tool tip for the scroll bar or the slider bar
        /// </summary>
        /// <param name="toolTip">reference to tooltip control being used</param>
        /// <param name="text">text to set tooltip to for the slider control</param>
        public void setScrollBarTooltip(ref ToolTip toolTip, string text)
        {
            toolTip.SetToolTip(sliderControl, text);
        }

        /// <summary>
        /// Set the tool tip for the textbox to
        /// </summary>
        /// <param name="toolTip">reference to tooltip control being used</param>
        /// <param name="text">text to set tooltip to for the textbox</param>
        public void setTextBoxTooltip(ref ToolTip toolTip, string text)
        {
            toolTip.SetToolTip(sliderValueTextBox, text);
        }

        /// <summary>
        /// Called when the slider bar is scrolled. If we are displaying as a float
        /// then figure out the corresponding float value based on the slider bar value.
        /// If not use the same value. Then update the textbox and trigger the callback
        /// </summary>
        /// <param name="sender">object that called the callback</param>
        /// <param name="e">arguments for the callback</param>
        private void sliderControl_Scroll(object sender, EventArgs e)
        {
            double value = 0.0;
            if (DisplayAsFloat)
                value = N_Math.scaleToRange(sliderControl.Value, sliderControl.Minimum, sliderControl.Maximum, Minimum, Maximum);
            else
                value = Convert.ToDouble(sliderControl.Value);

            mValue = value;
            updateTextBox();

            if (this.TypeValidText != null)
                this.TypeValidText(value);
        }

        /// <summary>
        /// Callback when the text is changed. If we are not displaying as a float then validate the
        /// text typed, while calling the appropriate callback and update the slider bar in real time.
        /// Otherwise we will wait for the user to hit enter.
        /// </summary>
        /// <param name="sender">object that called the callback</param>
        /// <param name="e">arguments for the callback</param>
        private void sliderValueTextBox_TextChanged(object sender, EventArgs e)
        {
            if (!mDisplayAsFloat)
                validateTextBox();
        }

        /// <summary>
        /// Callback when a key is pressed inside the textbox. We just care about the user
        /// hitting enter, so if they do check if the text in the textbox is valid and
        /// update the slider as well as calling the appropriate callback if it is or isn't
        /// </summary>
        /// <param name="sender">object that called the callback</param>
        /// <param name="e">arguments for the callback</param>
        private void sliderValueTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if(e.KeyCode == Keys.Enter)
                validateTextBox();
        }

        /// <summary>
        /// Callback when the textbox looses focus. Simply update the textbox with the
        /// old value.
        /// </summary>
        /// <param name="sender">object that called the callback</param>
        /// <param name="e">arguments for the callback</param>
        private void sliderValueTextBox_LostFocus(object sender, EventArgs e)
        {
            updateTextBox();
        }

        /// <summary>
        /// Check if the text in the textbox is valid, if we are handling things as a float check for a valid double
        /// if we aren't then handle it like an integer and check for that. Then call the appropriate callback
        /// depending if it is valid or not.
        /// </summary>
        private void validateTextBox()
        {
            bool isValid = false;
            double val;
            if (DisplayAsFloat)
            {
                //Check the user input
                isValid = double.TryParse(sliderValueTextBox.Text, out val);

            }
            else
            {
                int temp;
                //Check the user input
                isValid = int.TryParse(sliderValueTextBox.Text, out temp);

                val = Convert.ToDouble(temp);
            }

            //If the input was valid then update the slider else reset the textbox back to the
            //sliders value
            if (isValid)
            {
                onValidText(val);
            }
            else
                onInvalidText();
        }

        /// <summary>
        /// Event handler argument type for valid value callback
        /// </summary>
        /// <param name="value">valid value control is set to</param>
        public delegate void TypeValidTextEventHandler(double value);

        //[SRDescription("TypeValidText")]
        public event TypeValidTextEventHandler TypeValidText;
        //[SRDescription("TypeInvalidText")]
        public event EventHandler TypeInvalidText;

        /// <summary>
        /// Called when text entered in the textbox is valid
        /// </summary>
        /// <param name="value">valid value that the control is set to </param>
        private void onValidText(double value)
        {
            clampSliderValue(value);

            if (this.TypeValidText != null)
                this.TypeValidText(Value);
        }

        /// <summary>
        /// Called when text entered in the textbox is invalid
        /// </summary>
        private void onInvalidText()
        {
            updateTextBox();
            updateSliderBar();

            if (this.TypeInvalidText != null)
                this.TypeInvalidText(this, new EventArgs());
        }

        /// <summary>
        /// Get the range between our maximum and minimum value
        /// </summary>
        /// <returns>range between maximum and minimum value</returns>
        public double getRange()
        {
            return (Maximum - Minimum);
        }

        /// <summary>
        /// Get/set the value for the slider and textbox
        /// </summary>
        public double Value
        {
            get
            {
                return mValue;
            }

            set
            {
                clampSliderValue(value);
            }
        }

        /// <summary>
        /// Get/set maximum value for the slider and text box
        /// </summary>
        public double Maximum
        {
            get
            {
                return mMaximum;
            }

            set
            {
                if (value < Minimum)
                {
                    MessageBox.Show("Invalid Maximum: " + value.ToString() + " is < " + Minimum.ToString() + ".Maximum must be greater than Minimum.");
                }
                else
                {
                    mMaximum = value;
                    clampSliderValue(mValue);
                    updateSliderBarRange();
                }
            }
        }

        /// <summary>
        /// Get/set the minimum value for the slider and text box
        /// </summary>
        public double Minimum
        {
            get
            {
                return mMinimum;
            }

            set
            {
                if (value > Maximum)
                {
                    MessageBox.Show("Invalid Minimum: " + value.ToString() + " is > " + Maximum.ToString() + ".Maximum must be greater than Minimum.");
                    return;
                }
                else
                {
                    mMinimum = value;
                    clampSliderValue(sliderControl.Value);
                    updateSliderBarRange();
                }
            }
        }

        /// <summary>
        /// Get/Set small change amount for slider
        /// </summary>
        public double SmallChange
        {
            get
            {
                return mSmallChange;
            }

            set
            {
                mSmallChange = value;
                sliderControl.SmallChange = Convert.ToInt32(N_Math.scaleToRange(mSmallChange, Minimum, Maximum, sliderControl.Minimum, sliderControl.Maximum));
            }
        }

        /// <summary>
        /// Get/Set Large change amount for slider
        /// </summary>
        public double LargeChange
        {
            get
            {
                return mLargeChange;
            }

            set
            {
                mLargeChange = value;
                sliderControl.LargeChange = Convert.ToInt32(N_Math.scaleToRange(mLargeChange, Minimum, Maximum, sliderControl.Minimum, sliderControl.Maximum));
            }
        }

        /// <summary>
        /// Get/Set Tick Style for the slider
        /// </summary>
        public TickStyle TickStyle
        {
            get
            {
                return sliderControl.TickStyle;
            }

            set
            {
                sliderControl.TickStyle = value;
            }
        }

        /// <summary>
        /// Get/Set Tick Frequency for the slider
        /// </summary>
        public double TickFrequency
        {
            get
            {
                return mTickFrequency;
            }

            set
            {
                mTickFrequency = value;
                if (DisplayAsFloat)
                {
                    sliderControl.TickFrequency = Convert.ToInt32(N_Math.scaleToRange(mTickFrequency, Minimum, Maximum, sliderControl.Minimum, sliderControl.Maximum));
                }
                else
                    sliderControl.TickFrequency = Convert.ToInt32(mTickFrequency);
            }
        }

        /// <summary>
        /// Get/Set the control to handle everything as a float (if false then it will handle things like an integer)
        /// </summary>
        public bool DisplayAsFloat
        {
            get
            {
                return mDisplayAsFloat;
            }

            set
            {
                mDisplayAsFloat = value;
                updateSliderBarRange();
            }
        }

        /// <summary>
        /// Clamp the the value for the slider between its max and min and update the texbox
        /// </summary>
        /// <param name="val">value to set the slider to and to be clamped</param>
        private void clampSliderValue(double value)
        {
            mValue = N_Math.Clamp(value, Minimum, Maximum);
            updateSliderBar();
            updateTextBox();
        }

        /// <summary>
        /// Update the textbox with the value from the slider
        /// </summary>
        private void updateTextBox()
        {
            if(DisplayAsFloat)
                sliderValueTextBox.Text = mValue.ToString("0.00");
            else
                sliderValueTextBox.Text = mValue.ToString();
        }

        /// <summary>
        /// Update the slider bars min and max range based on how we are handling this control (float vs int)
        /// </summary>
        private void updateSliderBarRange()
        {
            if (DisplayAsFloat)
            {
                sliderControl.Maximum = SLIDER_RESOLUTION;
                sliderControl.Minimum = 0;
            }
            else
            {
                sliderControl.Maximum = Convert.ToInt32(Maximum);
                sliderControl.Minimum = Convert.ToInt32(Minimum);
            }

            mSliderBarRange = sliderControl.Maximum - sliderControl.Minimum;

            updateSliderBar();
        }

        /// <summary>
        /// Update the slider bar with the value we are storing
        /// </summary>
        private void updateSliderBar()
        {
            int position = 0;
            if (DisplayAsFloat)
            {
                double temp = N_Math.scaleToRange(Value, Minimum, Maximum, sliderControl.Minimum, sliderControl.Maximum);
                position = Convert.ToInt32(Math.Round(temp));
            }
            else
                position = Convert.ToInt32(Value);

            sliderControl.Value = position;
        }
    }
}
