﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NathanScott_Lib
{
    /// <summary>
    /// Status of measurement
    /// </summary>
    public enum MeasurementStatus
    {
        Disabled,
        Enabled,
        InProgress,
        Done
    };

    /// <summary>
    /// Holds a list of points and adds up the distance between them
    /// </summary>
    public class MeasurementHandler
    {
        /// <summary>
        /// Default constructor, starts off disable with no points and a distance of 0.0
        /// </summary>
        public MeasurementHandler()
        {
            mPoints = new List<Point2I>();
            mStatus = MeasurementStatus.Disabled;
            mDistance = 0.0;
        }

        /// <summary>
        /// Starts a measurement session (not required to call this)
        /// </summary>
        public void start()
        {
            if (mStatus == MeasurementStatus.Disabled)
            {
                mStatus = MeasurementStatus.Enabled;
            }
        }

        /// <summary>
        /// Add a click to the list (LMB) and update the distance
        /// </summary>
        /// <param name="point">Point to be logged and added to the distance</param>
        public void addPoint(Point2I point)
        {
            //If we were done we need to reset the click list
            //to start from scratch
            if (mStatus == MeasurementStatus.Done)
            {
                reset();
            }

            mStatus = MeasurementStatus.InProgress;

            //Need at least 2 points for the distance
            if (mPoints.Count > 0)
                mDistance += calculateDistance(mPoints[mPoints.Count - 1], point);

            //Add the click to the list
            mPoints.Add(point);
        }

        /// <summary>
        /// Edit the position of the last point in the list and update the distance correctly
        /// </summary>
        /// <param name="point">new position for last point</param>
        public void editLastPoint(Point2I point)
        {
            if (mStatus == MeasurementStatus.InProgress)
            {
                //Remove the last distance
                if (mPoints.Count > 1)
                    mDistance -= calculateDistance(mPoints[mPoints.Count - 2], mPoints[mPoints.Count - 1]);

                //change the point
                if (mPoints.Count > 0)
                    mPoints[mPoints.Count - 1] = point;

                //add the new distance
                if (mPoints.Count > 1)
                    mDistance += calculateDistance(mPoints[mPoints.Count - 2], mPoints[mPoints.Count - 1]);
            }
        }

        /// <summary>
        /// End the measurement
        /// </summary>
        public void end()
        {
            if (mStatus == MeasurementStatus.Enabled || mStatus == MeasurementStatus.InProgress)
            {
                mStatus = MeasurementStatus.Done;
            }
        }

        /// <summary>
        /// Disable measurement
        /// </summary>
        public void setDisabled()
        {
            reset();
            mStatus = MeasurementStatus.Disabled;
        }

        //Get the current measurement status
        public MeasurementStatus getStatus()
        {
            return mStatus;
        }

        /// <summary>
        /// Get a point at a given index
        /// </summary>
        /// <param name="index">index of point in list to get</param>
        /// <returns>the point at the given index or null if bad index</returns>
        public Point2I getPoint(int index)
        {
            if (index >= 0 && index < mPoints.Count)
                return mPoints[index];
            else
                return null;
        }

        /// <summary>
        /// Get the number of points in the current measurement session
        /// </summary>
        /// <returns>number of points in the list</returns>
        public int getCount()
        {
            return mPoints.Count;
        }

        /// <summary>
        /// Get the total distance between the points in this session
        /// </summary>
        /// <returns>total distance of all points</returns>
        public double getTotalDistance()
        {
            return mDistance;
        }

        /// <summary>
        /// Clear all the points and reset the distance
        /// </summary>
        private void reset()
        {
            if (mPoints.Count > 0)
                mPoints.Clear();

            mDistance = 0.0;
        }

        //----------- Calculating distance between clicks-----------------------------
        /// <summary>
        /// Calculate the distance from the first point to every point up to the last point
        /// </summary>
        /// <returns>distance from first point to every point up to last point</returns>
        private double calculateDistance()
        {
            return calculateDistance(0);
        }

        /// <summary>
        /// Calculate the distance from a start index to every point up to the last point
        /// </summary>
        /// <param name="startIndex">start index in list to calculate the distance from</param>
        /// <returns>distance from start index to every point up to last point</returns>
        private double calculateDistance(int startIndex)
        {
            return calculateDistance(startIndex, mPoints.Count - 1);
        }

        /// <summary>
        /// Calculate the distance from the start index to every point up to the end index
        /// </summary>
        /// <param name="startIndex">start index of distance calculation</param>
        /// <param name="endIndex">end index of distance calculation</param>
        /// <returns>distance from the start index to every point up to the end index</returns>
        private double calculateDistance(int startIndex, int endIndex)
        {
            double distance = 0.0;
            if (mPoints.Count > 1 && startIndex < endIndex && startIndex >= 0 && endIndex < mPoints.Count)
            {
                for (int i = startIndex, j = startIndex + 1; j <= endIndex; i++, j++)
                {
                    distance += calculateDistance(mPoints[i], mPoints[j]);
                }
            }

            return distance;
        }

        /// <summary>
        /// Calculate the distance between 2 points (returns absolute value)
        /// </summary>
        /// <param name="point1">first point</param>
        /// <param name="point2">second point</param>
        /// <returns>distance between the first and second point</returns>
        private double calculateDistance(Point2I point1, Point2I point2)
        {
            return Math.Abs(Math.Sqrt(Math.Pow(Convert.ToDouble(point2.x - point1.x), 2.0) + Math.Pow(Convert.ToDouble(point2.y - point1.y), 2.0)));
        }

        //----------------------Private Data Members----------------------------------------
        private List<Point2I> mPoints;      ///Keep track of our points
        private MeasurementStatus mStatus;  ///Keep track of the current measurement status
        private double mDistance;           ///Keep track of the total distance
    }
}
