﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using NathanScott_Lib;

namespace UWBGL_XNA_Lib13
{
    public class UWB_BoundingSphere : UWB_BoundingVolume
    {
        /// <summary>
        /// Default constructor
        /// </summary>
        public UWB_BoundingSphere()
        {
            mMin = Vector3.Zero;
            mMax = Vector3.Zero;
        }

        /// <summary>
        /// Create a bounding sphere with a given center and radius
        /// </summary>
        /// <param name="center">center point at which sphere will be</param>
        /// <param name="radius">radius of sphere</param>
        public UWB_BoundingSphere(Vector3 center, float radius)
        {
            setCenterAndRadius(center, radius);
        }

        /// <summary>
        /// Set the center and radius of the sphere (updating min, max)
        /// </summary>
        /// <param name="center">center point to move the sphere to</param>
        /// <param name="radius">new radius for the sphere</param>
        public virtual void setCenterAndRadius(Vector3 center, float radius)
        {
            mMin.X = center.X - radius;
            mMin.Y = center.Y - radius;
            mMin.Z = center.Z - radius;

            mMax.X = center.X + radius;
            mMax.Y = center.Y + radius;
            mMax.Z = center.Z + radius;
        }

        /// <summary>
        /// set the Radius of the sphere (min, max determined using radius and current center)
        /// </summary>
        /// <param name="radius">new radius to give the sphere</param>
        public virtual void setRadius(float radius)
        {
            Vector3 center = getCenter();

            mMin.X = center.X - radius;
            mMin.Y = center.Y - radius;
            mMin.Z = center.Z - radius;

            mMax.X = center.X + radius;
            mMax.Y = center.Y + radius;
            mMax.Z = center.Z + radius;
        }

        /// <summary>
        /// Get the bounding type (used in collision tests)
        /// </summary>
        /// <returns>bounding type</returns>
        public override eVolumeType getType()
        {
            return eVolumeType.sphere;
        }

        /// <summary>
        /// Test if this bounding object intersects with another
        /// </summary>
        /// <param name="other">other bounding volume to test with</param>
        /// <param name="pos">midpoint position at which the volumes intersect</param>
        /// <returns>true if volumes intersect, false if not</returns>
        public override bool intersects(UWB_BoundingVolume other, ref Vector3 pos)
        {
            eVolumeType vt = other.getType();

            bool isIntersecting = false;

            switch (vt)
            {
                case eVolumeType.box:
                    UWB_BoundingBox otherBox = other as UWB_BoundingBox;
                    isIntersecting = Nate_and_Scott_Collision.intersectBoxSphere(otherBox.getMin(), otherBox.getMax(), getCenter(), getRadius(), ref pos);
                    break;
                case eVolumeType.sphere:
                    UWB_BoundingSphere otherSphere = other as UWB_BoundingSphere;
                    isIntersecting = Nate_and_Scott_Collision.intersectSphereSphere(getCenter(), getRadius(), otherSphere.getCenter(), otherSphere.getRadius(), ref pos);
                    break;
            }


            return isIntersecting;
        }

        /// <summary>
        /// Get the radius of the sphere
        /// </summary>
        /// <returns>radius of the sphere</returns>
        public float getRadius()
        {
            return (mMax.X - mMin.X)/2f;
        }

        /// <summary>
        /// Test if point is inside the sphere
        /// </summary>
        /// <param name="testPoint">point or position to test</param>
        /// <returns>true if the point is inside the sphere, false if not</returns>
        public override bool containsPoint(Vector3 testPoint)
        {
            Vector3 center = getCenter();
            float radius = getRadius();
            return (Math.Pow(testPoint.X - center.X, 2) + Math.Pow(testPoint.Y - center.Y, 2) + Math.Pow(testPoint.Z - center.Z, 2) < Math.Pow(radius, 2));
        }

        /// <summary>
        /// Draw the sphere as a sphere
        /// </summary>
        /// <param name="drawHelper">draw helper that contains draw functions to wrap graphics API</param>
        public override void draw(UWB_DrawHelper drawHelper)
        {
            base.draw(drawHelper);

            if (drawHelper != null)
            {
                Vector3 center = getCenter();
                float radius = getRadius();
                drawHelper.drawCircle(center, radius);
            }
        }

        /// <summary>
        /// Make this volume invalid
        /// </summary>
        public void makeInvalid()
        {
            mMin = Vector3.Zero;
            mMax = new Vector3(-1, -1, -1);
        }

        /// <summary>
        /// Test if volume is valid
        /// </summary>
        /// <returns></returns>
        public bool isValid()
        {
            return (mMin.X <= mMax.X && mMin.Y <= mMax.Y && mMin.Z <= mMax.Z);
        }

        /// <summary>
        /// Resize the sphere to fit the bounding volume within it
        /// </summary>
        /// <param name="other">bounding volume to fit to</param>
        public override void add(UWB_BoundingVolume other)
        {
            if (other != null)
            {
                eVolumeType vt = other.getType();
                if (eVolumeType.sphere == vt)
                {
                    UWB_BoundingSphere sphere = other as UWB_BoundingSphere;
                    add(sphere);
                }
                else if (eVolumeType.box == vt)
                {
                    UWB_BoundingBox box = other as UWB_BoundingBox;
                    add(box);
                }
            }
        }

        /// <summary>
        /// Resize the sphere to fit the other sphere within it
        /// </summary>
        /// <param name="other">sphere to fit to</param>
        public void add(UWB_BoundingSphere sphere)
        {
            if (!sphere.isValid())
                return;

            if (!isValid())
                setCenterAndRadius(sphere.getCenter(), sphere.getRadius());
            else
            {
                mMin.X = mMin.X < sphere.mMin.X ? mMin.X : sphere.mMin.X;
                mMin.Y = mMin.Y < sphere.mMin.Y ? mMin.Y : sphere.mMin.Y;
                mMin.Z = mMin.Z < sphere.mMin.Z ? mMin.Z : sphere.mMin.Z;
                mMax.X = mMax.X > sphere.mMax.X ? mMax.X : sphere.mMax.X;
                mMax.Y = mMax.Y > sphere.mMax.Y ? mMax.Y : sphere.mMax.Y;
                mMax.Z = mMax.Z > sphere.mMax.Z ? mMax.Z : sphere.mMax.Z;
            }
        }

        public void add(UWB_BoundingBox box)
        {
            if (!box.isValid())
                return;

            if (!isValid())
                setCenterAndRadius(box.getCenter(), Vector3.Distance(box.getMax(), box.getMin())/2.0f);
            else
            {
                mMin.X = mMin.X < box.getMin().X ? mMin.X : box.getMin().X;
                mMin.Y = mMin.Y < box.getMin().Y ? mMin.Y : box.getMin().Y;
                mMin.Z = mMin.Z < box.getMin().Z ? mMin.Z : box.getMin().Z;
                mMax.X = mMax.X > box.getMax().X ? mMax.X : box.getMax().X;
                mMax.Y = mMax.Y > box.getMax().Y ? mMax.Y : box.getMax().Y;
                mMax.Z = mMax.Z > box.getMax().Z ? mMax.Z : box.getMax().Z;
            }
        }
    }
}
