﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace UWBGL_XNA_Lib13
{
    class UWB_BoundingBox_SphereList : UWB_BoundingBox
    {
        UWB_Array<UWB_BoundingVolume> mList;

        /// <summary>
        /// Default constructor
        /// </summary>
        public UWB_BoundingBox_SphereList()
        {
            mList = new UWB_Array<UWB_BoundingVolume>();
            mMin = Vector3.Zero;
            mMax = Vector3.Zero;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="UWB_BoundingBox_SphereList"/> class.
        /// </summary>
        /// <param name="min">The min.</param>
        /// <param name="max">The max.</param>
        public UWB_BoundingBox_SphereList(Vector3 min, Vector3 max)
            : this()
        {
            mMin = min;
            mMax = max;

            float width = max.X - min.X;
            float height = max.Y - min.Y;

            Vector3 center = new Vector3(((min.X + max.X) / 2), ((min.Y + max.Y) / 2), 0);

            float largerSideLength = Math.Max(width, height);
            float smallerSideLength = Math.Min(width, height);

            bool xDifferenceIsLarger = (largerSideLength == width);

            float diameter = smallerSideLength;
            float radius = smallerSideLength / 2;

            int numSpheres = 0;

            // prevent divide by zero
            if (largerSideLength != 0 && smallerSideLength != 0)
            {
                numSpheres = Convert.ToInt32(largerSideLength / smallerSideLength);
            }

            // use a line to keep track of the center line, and its orientation in space (slope, I guess).
            UWB_PrimitiveLine centerLine = new UWB_PrimitiveLine();

            if (xDifferenceIsLarger)
            {
                centerLine.setStartPoint(min.X, ((min.Y + max.Y) / 2), 0);
                centerLine.setEndPoint(max.X, ((min.Y + max.Y) / 2), 0);
            }
            else
            {
                centerLine.setStartPoint(((min.X + max.X) / 2), min.Y, 0);
                centerLine.setEndPoint(((min.X + max.X) / 2), max.Y, 0);
            }

            List<Vector3> sphereCenters = new List<Vector3>();

            // start at one end of the line and add spheres along it until we're done
            // (might have to start just beyond the line so everything is even...find that difference here)

            float totalLengthOfSpheres = numSpheres * (radius * 2);
            float totalLengthOfCenterLine = centerLine.getLength();
            float startCorrectionValue = (totalLengthOfCenterLine - totalLengthOfSpheres) / 2; // evenly over or short on both ends.

            //float slopeOfLine = 0;

            //if (!(centerLine.getEndPoint().X - centerLine.getStartPoint().X == 0))
            //{
            //    slopeOfLine = (centerLine.getEndPoint().Y - centerLine.getStartPoint().Y) / (centerLine.getEndPoint().X - centerLine.getStartPoint().X);
            //}

            // you know what...I don't need the slope of the line...all our rectangles are straight up and down...
            // let's find the start point....
            Vector3 startPoint = Vector3.Zero;

            if (xDifferenceIsLarger)
            {
                startPoint = new Vector3(min.X + startCorrectionValue + radius, ((min.Y + max.Y) / 2), 0);
            }
            else
            {
                startPoint = new Vector3(((min.X + max.X) / 2), min.Y + startCorrectionValue + radius, 0);
            }

            Vector3 currentPoint = startPoint;

            for (int i = 0; i < numSpheres; i++)
            {
                sphereCenters.Add(currentPoint);

                if (xDifferenceIsLarger)
                {
                    currentPoint = new Vector3(currentPoint.X + (radius * 2), currentPoint.Y, 0);
                }
                else
                {
                    currentPoint = new Vector3(currentPoint.X, currentPoint.Y + (radius * 2), 0);
                }
            }

            if ((sphereCenters != null) && (sphereCenters.Count > 0))
            {
                if (mList == null)
                {
                    mList = new UWB_Array<UWB_BoundingVolume>();
                }

                foreach (Vector3 sphereCenter in sphereCenters)
                {
                    mList.append(new UWB_BoundingSphere(sphereCenter, radius));
                }
            }
        }

        /// <summary>
        /// Test if this bounding object intersects with another
        /// </summary>
        /// <param name="other">other bounding volume to test with</param>
        /// <param name="pos">midpoint position at which the volumes intersect</param>
        /// <returns>true if volumes intersect, false if not</returns>
        public override bool intersects(UWB_BoundingVolume other, ref Vector3 pos)
        {
            bool isIntersecting = false;

            // check each sphere to see if it intersects
            if ((mList != null) && (mList.count() > 0))
            {
                for (int i = 0; i < mList.count(); i++)
                {
                    if (mList.getItem(i).intersects(other, ref pos))
                    {
                        isIntersecting = true;
                        break;
                    }
                }
            }

            return isIntersecting;
        }

        /// <summary>
        /// Test if point is inside the the volume
        /// </summary>
        /// <param name="testPoint">point or position to test</param>
        /// <returns>true if the point is inside the volume list, false if not</returns>
        public override bool containsPoint(Vector3 testPoint)
        {
            bool isIntersecting = false;

            // check each sphere to see if it intersects
            if ((mList != null) && (mList.count() > 0))
            {
                for (int i = 0; i < mList.count(); i++)
                {
                    if (mList.getItem(i).containsPoint(testPoint))
                    {
                        isIntersecting = true;
                        break;
                    }
                }
            }

            return isIntersecting;
        }

        /// <summary>
        /// Draw the volume
        /// </summary>
        /// <param name="drawHelper">draw helper that contains draw functions to wrap graphics API</param>
        public override void draw(UWB_DrawHelper drawHelper)
        {
            if ((drawHelper != null) && (mList != null) && (mList.count() > 0))
            {
                for (int i = 0; i < count(); i++)
                {
                    UWB_BoundingVolume volume = mList.getItem(i);
                    volume.draw(drawHelper);
                }
            }
        }

        /// <summary>
        /// Returns the count within this list.
        /// </summary>
        /// <returns></returns>
        public int count()
        {
            int output = 0;

            if ((mList != null) && (mList.count() > 0))
            {
                output = mList.count();
            }

            return output;
        }

        /// <summary>
        /// Returns the volume at the specified index.
        /// </summary>
        /// <param name="index">The index.</param>
        /// <returns></returns>
        public UWB_BoundingVolume volumeAt(int index)
        {
            UWB_BoundingVolume output = null;

            if ((mList != null) && (mList.count() > 0))
            {
                if (index < 0 || index >= mList.count())
                {
                    return null;
                }

                output = mList.getItem(index);
            }

            return output;
        }

        /// <summary>
        /// Finds the index of the volume.
        /// </summary>
        /// <param name="volume">The volume.</param>
        /// <returns></returns>
        public int findVolumeIndex(UWB_BoundingVolume volume)
        {
            if ((volume != null) && (mList != null) && (mList.count() > 0))
            {
                for (int i = 0; i < count(); i++)
                {
                    UWB_BoundingVolume testVolume = mList.getItem(i);
                    if (volume == testVolume)
                        return i;
                }
            }

            return -1;
        }

        /// <summary>
        /// Appends the specified volume.
        /// </summary>
        /// <param name="volume">The volume.</param>
        public void append(UWB_BoundingVolume volume)
        {
            if (mList == null)
            {
                mList = new UWB_Array<UWB_BoundingVolume>();
            }

            mList.append(volume);
        }

        /// <summary>
        /// Deletes the volume at.
        /// </summary>
        /// <param name="index">The index.</param>
        public void deleteVolumeAt(int index)
        {
            if ((mList != null) && (mList.count() > 0))
            {
                mList.deleteItem(index);
            }
        }
    }
}
