using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using NathanScott_Lib;
using System.Xml.Serialization;

namespace UWBGL_XNA_Lib13
{
	public class UWB_BoundingVolume
	{
		protected Vector3 mMin;
		protected Vector3 mMax;

		public enum eVolumeType { box, sphere };

		public virtual eVolumeType getType() 
		{ 
			return eVolumeType.box; 
		}

        public virtual void setCenter(Vector3 center)
        {
            Vector3 oldCenter = getCenter();
            Vector3 delta = center - oldCenter;

            mMin += delta;
            mMax += delta;
        }

		public virtual Vector3 getCenter() 
		{
            Vector3 c = (mMin + mMax) / 2f;
            return c;
		}

        public Vector3 getMin()
        {
            return mMin;
        }

        public Vector3 getMax()
        {
            return mMax;
        }

		public virtual void add(UWB_BoundingVolume other) { }

		public virtual bool containsPoint(Vector3 testPoint) 
		{
			return false;
		}

		public virtual bool intersects(UWB_BoundingVolume other, ref Vector3 pos) 
		{
			return false;
		}

        public virtual bool isOutside(UWB_BoundingVolume other)
        {
            return Nate_and_Scott_Collision.isOutsideBox(mMin, mMax, other.getMin(), other.getMax());
        }

		public virtual void draw(UWB_DrawHelper drawHelper)
        {
            if (drawHelper != null)
            {
                drawHelper.enableBlending(false);
                drawHelper.enableTexture(false);
            }
        }
	}

    [Serializable]
	public class UWB_BoundingBox : UWB_BoundingVolume
	{
		public UWB_BoundingBox()
		{
			mMin = Vector3.Zero;
			mMax = Vector3.Zero;
		}

		public UWB_BoundingBox(Vector3 corner1, Vector3 corner2)
		{
			setCorners(corner1, corner2);
		}

		public UWB_BoundingBox(Vector3 center, float width, float height, float depth)
		{
			mMin.X = center.X - width / 2f;
			mMin.Y = center.Y - height / 2f;
			mMin.Z = center.Z - depth / 2f;
			mMax.X = center.X + width / 2f;
			mMax.Y = center.Y + height / 2f;
			mMax.Z = center.Z + depth / 2f;
		}

		public void setCorners(Vector3 corner1, Vector3 corner2)
		{
			mMin.X = corner1.X < corner2.X ? corner1.X : corner2.X;
			mMin.Y = corner1.Y < corner2.Y ? corner1.Y : corner2.Y;
			mMin.Z = corner1.Z < corner2.Z ? corner1.Z : corner2.Z;

			mMax.X = corner1.X > corner2.X ? corner1.X : corner2.X;
			mMax.Y = corner1.Y > corner2.Y ? corner1.Y : corner2.Y;
			mMax.Z = corner1.Z > corner2.Z ? corner1.Z : corner2.Z;
		}

		public void moveToCenter(Vector3 newCenter)
		{
			setCenter(newCenter);
		}

		public override eVolumeType getType()
		{
			return eVolumeType.box; 
		}

        public override bool intersects(UWB_BoundingVolume other, ref Vector3 pos)
		{
			eVolumeType vt = other.getType();

            bool isIntersecting = false;

            switch (vt)
            {
                case eVolumeType.box:
                    UWB_BoundingBox otherBox = other as UWB_BoundingBox;
                    isIntersecting = Nate_and_Scott_Collision.intersectBoxBox(mMin, mMax, otherBox.getMin(), otherBox.getMax(), ref pos);
                    break;
                case eVolumeType.sphere:
                    UWB_BoundingSphere otherSphere = other as UWB_BoundingSphere;
                    isIntersecting = Nate_and_Scott_Collision.intersectBoxSphere(mMin, mMax, otherSphere.getCenter(), otherSphere.getRadius(), ref pos);
                    break;
            }


            return isIntersecting;
		}

		public float getWidth()
		{
			return mMax.X - mMin.X;
		}

		public float getHeight()
		{
			return mMax.Y - mMin.Y;
		}

		public float getDepth()
		{
			return mMax.Z - mMin.Z;
		}

        public void setWidth(float width)
        {
            Vector3 center = getCenter();

            mMax.X = center.X + width / 2;
            mMin.X = center.X - width / 2;
        }

        public void setHeight(float hieght)
        {
            Vector3 center = getCenter();

            mMax.Y = center.Y + hieght / 2;
            mMin.Y = center.Y - hieght / 2;
        }

        public void setDepth(float depth)
        {
            Vector3 center = getCenter();

            mMax.Z = center.Z + depth / 2;
            mMin.Z = center.Z - depth / 2;
        }
		
		public override bool containsPoint(Vector3 testPoint)
		{
			bool xInside = testPoint.X >=  mMin.X && testPoint.X <= mMax.X;
			if (!xInside)
				return false;

			bool yInside = testPoint.Y >= mMin.Y && testPoint.Y <= mMax.Y;
			if (!yInside)
				return false;

			bool zInside = testPoint.Z >= mMin.Z && testPoint.Z <= mMax.Z;
			if (!zInside)
				return false;

			return true;
		}

		public override void draw(UWB_DrawHelper drawHelper)
		{
            base.draw(drawHelper);

			if (drawHelper != null)
			{
                drawHelper.drawRectangle(mMin, mMax);
			}
		}

		public void makeInvalid()
		{
			mMin = Vector3.Zero;
			mMax = new Vector3(-1, -1, -1);
		}

		public bool isValid()
		{
			return (mMin.X <= mMax.X && mMin.Y <= mMax.Y && mMin.Z <= mMax.Z);
		}

		public override void add(UWB_BoundingVolume other)
		{
			if(other != null)
			{
				eVolumeType vt = other.getType();
				if(eVolumeType.box == vt)
				{
					UWB_BoundingBox box = other as UWB_BoundingBox;
					add(box);
				}
                else if (eVolumeType.sphere == vt)
                {
                    UWB_BoundingSphere sphere = other as UWB_BoundingSphere;
                    float width = sphere.getRadius() * 2;
                    UWB_BoundingBox box = new UWB_BoundingBox(sphere.getCenter(), width, width, 0.0f);
                    add(box);
                }
			}
		}

		public void add(UWB_BoundingBox box)
		{
			if (!box.isValid())
				return;

			if (!isValid())
				setCorners(box.getMax(), box.getMin());
			else
			{
				mMin.X = mMin.X < box.mMin.X ? mMin.X : box.mMin.X;
				mMin.Y = mMin.Y < box.mMin.Y ? mMin.Y : box.mMin.Y;
				mMin.Z = mMin.Z < box.mMin.Z ? mMin.Z : box.mMin.Z;
				mMax.X = mMax.X > box.mMax.X ? mMax.X : box.mMax.X;
				mMax.Y = mMax.Y > box.mMax.Y ? mMax.Y : box.mMax.Y;
				mMax.Z = mMax.Z > box.mMax.Z ? mMax.Z : box.mMax.Z;
			}
        }

        #region Public properties for XML serialization

        [XmlElement("MinPoint")]
        public Vector3 MinXML
        {
            get
            {
                return this.mMin;
            }
            set
            {
                this.mMin = value;
            }
        }

        [XmlElement("MaxPoint")]
        public Vector3 MaxXML
        {
            get
            {
                return this.mMax;
            }
            set
            {
                this.mMax = value;
            }
        }

        #endregion
    }
}
