using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Runtime.Serialization;
using System.Xml.Serialization;

namespace UWBGL_XNA_Lib13
{
	public class UWB_SceneNode
	{
		protected string mName;
		protected bool mPivotVisible;
		protected UWB_XFormInfo mXFormInfo;
		protected UWB_Array<UWB_SceneNode> mChildNodes;
		protected UWB_Primitive mPrimitive;
        protected Vector3 mVelocity;

        public UWB_SceneNode(string name)
        {
            mName = name;
            Init();
        }

		public UWB_SceneNode()
		{
            mName = "SceneNode";
            Init();
		}

        private void Init()
        {
            mVelocity = new Vector3(0, 0, 0);
            mPrimitive = null;
            mPivotVisible = false;

            mXFormInfo = new UWB_XFormInfo();
            mChildNodes = new UWB_Array<UWB_SceneNode>();
        }

		public void draw(eLevelofDetail lod, UWB_DrawHelper drawHelper)
		{
			drawHelper.pushModelTransform();
			mXFormInfo.setupModelStack(drawHelper);

			if (mPrimitive != null)
			{
				mPrimitive.draw(lod, drawHelper);
			}

			int count = mChildNodes.count();
			for (int i = 0; i < count; i++)
			{
				mChildNodes.getItem(i).draw(lod, drawHelper);
			}

			if (mPivotVisible)
			{
				mXFormInfo.drawPivot(drawHelper, 0.2f);
			}
			drawHelper.popModelTransform();
		}

		public void setPrimitive(UWB_Primitive primitive)
		{
			mPrimitive = primitive;
		}

		public UWB_Primitive getPrimitive()
		{
			return mPrimitive;
		}

		public void insertChildNode(UWB_SceneNode childNode)
		{
			mChildNodes.append(childNode);
		}

        public bool removeChildNode(UWB_SceneNode childNode)
        {
            return mChildNodes.deleteItem(childNode);
        }

		public int numChildren()
		{
			return mChildNodes.count();
		}

		public UWB_SceneNode getChildNode(int index)
		{
			if (index < 0 || index >= numChildren())
			{
				return null;
			}

			return mChildNodes.getItem(index);
		}

        /// <summary>
        /// Search for the node with the given name (finds first node with that name, so two nodes shouldn't have the same name)
        /// </summary>
        /// <param name="name">name to look for</param>
        /// <param name="caseSensitive">whether name check should be case sensitive or not</param>
        /// <param name="recursive">whether to recursively look for name (all children and sub-children)</param>
        /// <returns>reference to first node with that name, null if not found</returns>
        public UWB_SceneNode findNode(string name, bool caseSensitive, bool recursive)
        {
            if (mName.Equals(name, caseSensitive ? StringComparison.Ordinal : StringComparison.OrdinalIgnoreCase))
            {
                return this;
            }
            else
            {
                for (int i = 0; i < numChildren(); i++)
                {
                    UWB_SceneNode testNode = mChildNodes.getItem(i);

                    if (!recursive && testNode.getName().Equals(name, caseSensitive ? StringComparison.Ordinal : StringComparison.OrdinalIgnoreCase))
                    {
                        return testNode;
                    }
                    else if(recursive)
                    {
                        UWB_SceneNode node = testNode.findNode(name, caseSensitive, recursive);
                        if (node != null)
                            return node;
                    }
                }
            }

            return null;
        }

        /// <summary>
        /// Recursively search for the node with the given name (finds first node with that name, so two nodes shouldn't have the same name)
        /// </summary>
        /// <param name="name">name to look for</param>
        /// <param name="caseSensitive">whether name check should be case sensitive or not</param>
        /// <returns>reference to first node with that name, null if not found</returns>
        public UWB_SceneNode findNode(string name, bool caseSensitive)
        {
            return findNode(name, caseSensitive, true);
        }

        /// <summary>
        /// Recursively search for the node with the given case insensitive name (finds first node with that name, so two nodes shouldn't have the same name)
        /// </summary>
        /// <param name="name">case insensitive name to look for</param>
        /// <returns>reference to first node with that name, null if not found</returns>
        public UWB_SceneNode findNode(string name)
        {
            return findNode(name, false, true);
        }


		public UWB_XFormInfo getXFormInfo()
		{
			return mXFormInfo;
		}

		public void setXFormInfo(UWB_XFormInfo xForm)
		{
			mXFormInfo = xForm;
		}

        public void resetXFormInfo()
        {
            mXFormInfo.toIdentity();
            mXFormInfo.setPivot(Vector3.Zero);
        }

		public void setPivotVisible(bool visible)
		{
			mPivotVisible = visible;
		}

		public bool isPivotVisible()
		{
			return mPivotVisible;
		}

        public String getName()
        {
            return mName;
        }

        public void setVelocity(Vector3 v)
        {
            mVelocity = v;
        }
        public Vector3 getVelocity()
        {
            return mVelocity;
        }
        public virtual void moveNodeByVelocity(float elapsedSecond)
        {
            Vector3 translate = mXFormInfo.getTranslation();
            translate += (mVelocity * elapsedSecond);
            mXFormInfo.setTranslation(translate);
        }

        /// <summary>
        /// Adds the or subtract rotation.
        /// </summary>
        /// <param name="main">The main.</param>
        /// <param name="oneToAdd">The one to add.</param>
        /// <param name="add">if set to <c>true</c> [add].</param>
        public void AddOrSubtractRotation(UWB_XFormInfo main, UWB_XFormInfo oneToAdd, bool add)
        {
            if (add)
            {
                main.setRotationDegrees(main.getRotationInDegrees() + oneToAdd.getRotationInDegrees());
            }
            else
            {
                main.setRotationDegrees(main.getRotationInDegrees() - oneToAdd.getRotationInDegrees());
            }
        }

        public UWB_BoundingBox getBounds(UWB_DrawHelper helper, bool bDraw)
        {
            return getBounds(helper, false, bDraw);
        }

        public UWB_BoundingBox getBounds(UWB_DrawHelper helper, bool ignoreChildren, bool bDraw)
        {
            UWB_BoundingBox box = new UWB_BoundingBox();
            box.makeInvalid();
            if (mPrimitive != null)
                box.add(mPrimitive.getBoundingVolume(eLevelofDetail.lodLow));

            helper.pushModelTransform();
            {
                mXFormInfo.setupModelStack(helper);
                //Draw the box for debugging
                if (bDraw)
                    box.draw(helper);
                Vector3 minPt = box.getMin();
                Vector3 maxPt = box.getMax();
                Vector3 pt1 = new Vector3(minPt.X, minPt.Y, minPt.Z);
                Vector3 pt2 = new Vector3(maxPt.X, minPt.Y, minPt.Z);
                Vector3 pt3 = new Vector3(maxPt.X, maxPt.Y, minPt.Z);
                Vector3 pt4 = new Vector3(minPt.X, maxPt.Y, minPt.Z);
                Vector3 pt5 = new Vector3(minPt.X, minPt.Y, maxPt.Z);
                Vector3 pt6 = new Vector3(maxPt.X, minPt.Y, maxPt.Z);
                Vector3 pt7 = new Vector3(maxPt.X, maxPt.Y, maxPt.Z);
                Vector3 pt8 = new Vector3(minPt.X, maxPt.Y, maxPt.Z);
                helper.transformPoint(ref pt1);
                helper.transformPoint(ref pt2);
                helper.transformPoint(ref pt3);
                helper.transformPoint(ref pt4);
                helper.transformPoint(ref pt5);
                helper.transformPoint(ref pt6);
                helper.transformPoint(ref pt7);
                helper.transformPoint(ref pt8);

                box.makeInvalid();
                box.add(new UWB_BoundingBox(pt1, pt2));
                box.add(new UWB_BoundingBox(pt3, pt4));
                box.add(new UWB_BoundingBox(pt5, pt6));
                box.add(new UWB_BoundingBox(pt7, pt8));

                if (ignoreChildren == false)
                {
                    int count = mChildNodes.count();
                    for (int i = 0; i < count; i++)
                        box.add(mChildNodes.getItem(i).getBounds(helper, bDraw));
                }
            }
            helper.popModelTransform();

            return box;
        }

        /// <summary>
        /// Gets the specified node bounds.
        /// </summary>
        /// <param name="pSearchNode">The search node.</param>
        /// <param name="box">The box.</param>
        /// <param name="helper">The helper.</param>
        /// <param name="level">The level.</param>
        /// <param name="bDraw">if set to <c>true</c> [draw].</param>
        /// <returns></returns>
        protected bool getNodeBoundsHelper(UWB_SceneNode pSearchNode, bool ignoreChildren, ref UWB_BoundingBox box,
                                           UWB_DrawHelper helper, int level, bool bDraw)
        {
            bool found = false;
            if (Object.ReferenceEquals(this, pSearchNode))
            {
                box = this.getBounds(helper, ignoreChildren, bDraw);
                found = true;
            }
            else
            {
                helper.pushModelTransform();
                {
                    level++;
                    this.getXFormInfo().setupModelStack(helper);

                    int count = this.numChildren();
                    for (int i = 0; i < count; i++)
                    {
                        UWB_SceneNode pChildNode = this.getChildNode(i);
                        if (pChildNode.getNodeBoundsHelper(pSearchNode, ignoreChildren, ref box, helper, level, bDraw))
                        {
                            found = true;
                            break;
                        }
                    }
                    level--;
                }
                helper.popModelTransform();
            }

            if (0 == level && found && bDraw)
            {
                helper.resetAttributes();
                helper.setColor1(new Color(0, 0, 255));
                helper.setShadeMode(eShadeMode.smFlat);
                helper.setFillMode(eFillMode.fmWireframe);
                box.draw(helper);
            }

            return found;
        }

        /// <summary>
        /// Gets the node bounds.
        /// </summary>
        /// <param name="pSearchNode">The node to search for.</param>
        /// <param name="box">The box.</param>
        /// <param name="helper">The helper.</param>
        /// <param name="bDraw">if set to <c>true</c> [draw].</param>
        /// <returns></returns>
        public bool getNodeBounds(UWB_SceneNode pSearchNode, ref UWB_BoundingBox box,
                                  UWB_DrawHelper helper, bool bDraw)
        {
            return getNodeBoundsHelper(pSearchNode, false, ref box, helper, 0, bDraw);
        }

        public bool getNodeBounds(UWB_SceneNode pSearchNode, bool ignoreChildren, ref UWB_BoundingBox box,
                                  UWB_DrawHelper helper, bool bDraw)
        {
            return getNodeBoundsHelper(pSearchNode, ignoreChildren, ref box, helper, 0, bDraw);
        }

        /// <summary>
        /// Gets the primitive bounds helper.
        /// </summary>
        /// <param name="primitive">The primitive.</param>
        /// <param name="helper">The helper.</param>
        /// <param name="bDraw">if set to <c>true</c> [draw].</param>
        /// <returns></returns>
        protected UWB_BoundingVolume getPrimitiveBoundsHelper(UWB_Primitive primitive, UWB_DrawHelper helper, bool bDraw)
        {
            UWB_BoundingVolume output = null;

            if ((primitive != null) && (helper != null))
            {
                UWB_BoundingVolume temp = primitive.getBoundingVolume(eLevelofDetail.lodMed);

                // must check this first
                if (temp is UWB_BoundingBox_SphereList)
                {
                    Vector3 min = primitive.getBoundingVolume(eLevelofDetail.lodMed).getMin();
                    Vector3 max = primitive.getBoundingVolume(eLevelofDetail.lodMed).getMax();

                    UWB_BoundingBox_SphereList sphereList = new UWB_BoundingBox_SphereList(min, max);

                    helper.pushModelTransform();
                    {
                        mXFormInfo.setupModelStack(helper);

                        // Draw the bounding volume for debugging.
                        if (bDraw)
                        {
                            sphereList.draw(helper);
                        }

                        Vector3 pt1 = new Vector3(min.X, min.Y, min.Z);
                        Vector3 pt2 = new Vector3(max.X, min.Y, min.Z);
                        Vector3 pt3 = new Vector3(max.X, max.Y, min.Z);
                        Vector3 pt4 = new Vector3(min.X, max.Y, min.Z);
                        Vector3 pt5 = new Vector3(min.X, min.Y, max.Z);
                        Vector3 pt6 = new Vector3(max.X, min.Y, max.Z);
                        Vector3 pt7 = new Vector3(max.X, max.Y, max.Z);
                        Vector3 pt8 = new Vector3(min.X, max.Y, max.Z);
                        helper.transformPoint(ref pt1);
                        helper.transformPoint(ref pt2);
                        helper.transformPoint(ref pt3);
                        helper.transformPoint(ref pt4);
                        helper.transformPoint(ref pt5);
                        helper.transformPoint(ref pt6);
                        helper.transformPoint(ref pt7);
                        helper.transformPoint(ref pt8);

                        // Rebuild the sphere list based on transformed points.
                        sphereList.makeInvalid();
                        sphereList.add(new UWB_BoundingBox(pt1, pt2));
                        sphereList.add(new UWB_BoundingBox(pt3, pt4));
                        sphereList.add(new UWB_BoundingBox(pt5, pt6));
                        sphereList.add(new UWB_BoundingBox(pt7, pt8));
                    }
                    helper.popModelTransform();

                    return sphereList;
                }
                else if ((temp is UWB_BoundingSphere) || (temp is UWB_BoundingBox))
                {
                    // TODO: NOTE: I only kept it this way with multiple returns to keep what we had before when it was just Sphere.
                    // If it all works, it should be refactored.
                    UWB_BoundingSphere sphere = new UWB_BoundingSphere();
                    sphere.makeInvalid();
                    sphere.add(primitive.getBoundingVolume(eLevelofDetail.lodLow));

                    helper.pushModelTransform();
                    {
                        mXFormInfo.setupModelStack(helper);

                        //Draw the box for debugging
                        if (bDraw)
                            sphere.draw(helper);

                        //Get the two points that make up a circle
                        Vector3 center = sphere.getCenter();
                        Vector3 outerPt = center;
                        outerPt.X += sphere.getRadius();

                        //Transform those points
                        helper.transformPoint(ref center);
                        helper.transformPoint(ref outerPt);

                        //Rebuild the sphere based on the transformed points
                        sphere.makeInvalid();
                        sphere.add(new UWB_BoundingSphere(center, Vector3.Distance(center, outerPt)));
                    }
                    helper.popModelTransform();

                    return sphere;
                }
            }
            return output;
        }

        /// <summary>
        /// Gets the node's primitive's bounds.
        /// </summary>
        /// <param name="pSearchPrimitive">The search primitive.</param>
        /// <param name="boundingVolume">The sphere.</param>
        /// <param name="helper">The helper.</param>
        /// <param name="level">The level.</param>
        /// <param name="bDraw">if set to <c>true</c> [draw].</param>
        /// <returns></returns>
        protected bool getNodePrimitiveBoundsHelper(    UWB_Primitive pSearchPrimitive,
                                                    ref UWB_BoundingVolume boundingVolume,
                                                        UWB_DrawHelper helper,
                                                        int level,
                                                        bool bDraw)
        {
            bool found = false;
            if (getPrimitive() != null && Object.ReferenceEquals(getPrimitive(), pSearchPrimitive))
            {
                //if the primitive list or single primitive is the one we want use it
                boundingVolume = getPrimitiveBoundsHelper(getPrimitive(), helper, bDraw);

                found = true;
            }
            else if (getPrimitive() != null && getPrimitive() is UWB_PrimitiveList)
            {
                //if there is a primitive list (probably is) and it wasn't it, then search for the primitive in the list.
                UWB_PrimitiveList pList = getPrimitive() as UWB_PrimitiveList;
                for (int i = 0; i < pList.count(); i++)
                {
                    UWB_Primitive testPrimitive = pList.primitiveAt(i);
                    if (Object.ReferenceEquals(testPrimitive, pSearchPrimitive))
                    {
                        boundingVolume = getPrimitiveBoundsHelper(testPrimitive, helper, bDraw);

                        found = true;
                        break;
                    }
                }
            }
            
            //If we didn't find it in this nodes primitive list then look in this nodes children
            if(!found)
            {
                helper.pushModelTransform();
                {
                    level++;
                    this.getXFormInfo().setupModelStack(helper);

                    int count = this.numChildren();
                    for (int i = 0; i < count; i++)
                    {
                        UWB_SceneNode pChildNode = this.getChildNode(i);
                        if (pChildNode.getNodePrimitiveBoundsHelper(pSearchPrimitive, ref boundingVolume, helper, level, bDraw))
                        {
                            found = true;
                            break;
                        }
                    }
                    level--;
                }
                helper.popModelTransform();
            }
            if (0 == level && found && bDraw)
            {
                helper.resetAttributes();
                helper.setColor1(new Color(0, 0, 255));
                helper.setShadeMode(eShadeMode.smFlat);
                helper.setFillMode(eFillMode.fmWireframe);
                boundingVolume.draw(helper);
            }

            return found;
        }

        /// <summary>
        /// Gets the node's primitive's bounds.
        /// </summary>
        /// <param name="pSearchPrimitive">The search primitive.</param>
        /// <param name="boundingVolume">The sphere.</param>
        /// <param name="helper">The helper.</param>
        /// <param name="bDraw">if set to <c>true</c> [draw].</param>
        /// <returns></returns>
        public bool getNodePrimitiveBounds(     UWB_Primitive pSearchPrimitive,
                                            ref UWB_BoundingVolume boundingVolume,
                                                UWB_DrawHelper helper,
                                                bool bDraw)
        {
            return getNodePrimitiveBoundsHelper(pSearchPrimitive, ref boundingVolume, helper, 0, bDraw);
        }

        //#region Public properties for XML serialization

        //[XmlElement("Name")]
        //public string NameXML
        //{
        //    get
        //    {
        //        return this.mName;
        //    }
        //    set
        //    {
        //        this.mName = value;
        //    }
        //}

        //[XmlElement("PivotVisible")]
        //public bool PivotVisibleXML
        //{
        //    get
        //    {
        //        return this.mPivotVisible;
        //    }
        //    set
        //    {
        //        this.mPivotVisible = value;
        //    }
        //}

        //[XmlElement("XFormInfo")]
        //public UWB_XFormInfo XFormInfoXML
        //{
        //    get
        //    {
        //        return this.mXFormInfo;
        //    }
        //    set
        //    {
        //        this.mXFormInfo = value;
        //    }
        //}

        //[XmlElement("ChildNodes")]
        //public UWB_Array<UWB_SceneNode> ChildNodesXML
        //{
        //    get
        //    {
        //        return this.mChildNodes;
        //    }
        //    set
        //    {
        //        this.mChildNodes = value;
        //    }
        //}

        //[XmlElement("Primitive")]
        //public UWB_Primitive PrimitiveXML
        //{
        //    get
        //    {
        //        return this.mPrimitive;
        //    }
        //    set
        //    {
        //        this.mPrimitive = value;
        //    }
        //}

        //public Vector3 Velocity
        //{
        //    get
        //    {
        //        return getVelocity();
        //    }
        //    set
        //    {
        //        setVelocity(value);
        //    }
        //}

        //#endregion
    }
}
