﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Media;
using UWBGL_XNA_Lib13;
using NathanScott_Lib;

namespace UWBGL_XNA_Lib13.Audio
{
    public class AudioManager
    {
        private Dictionary<SoundEffect, SoundEffectInstance> mSoundEffectInstances = new Dictionary<SoundEffect, SoundEffectInstance>(); 
        private ContentManager mContent = null;

        /// <summary>
        /// Initializes a new instance of the <see cref="AudioManager"/> class.
        /// </summary>
        public AudioManager(ContentManager content)
        {
            mContent = content;
        }

        /// <summary>
        /// Plays a sound based on specified volume, pitch, and panning (doesn't loop sound)
        /// </summary>
        /// <param name="soundName">The file name if the sound effect you wish to play (should be in the Content\Sounds folder).</param>
        /// <param name="volume">Volume, ranging from 0.0f (silence) to 1.0f (full volume). 1.0f is full volume relative to SoundEffect.MasterVolume.</param>
        /// <param name="pitch">Pitch adjustment, ranging from -1.0f (down one octave) to 1.0f (up one octave).  0.0f is unity (normal) pitch.</param>
        /// <param name="pan">Panning, ranging from -1.0f (full left) to 1.0f (full right). 0.0f is centered.</param>
        public void playSound(string soundName, float volume, float pitch, float pan)
        {
            playSound(soundName, volume, pitch, pan, false);
        }

        /// <summary>
        /// Plays a sound based on specified volume, pitch, and panning.
        /// </summary>
        /// <param name="soundName">The file name if the sound effect you wish to play (should be in the Content\Sounds folder).</param>
        /// <param name="volume">Volume, ranging from 0.0f (silence) to 1.0f (full volume). 1.0f is full volume relative to SoundEffect.MasterVolume.</param>
        /// <param name="pitch">Pitch adjustment, ranging from -1.0f (down one octave) to 1.0f (up one octave).  0.0f is unity (normal) pitch.</param>
        /// <param name="pan">Panning, ranging from -1.0f (full left) to 1.0f (full right). 0.0f is centered.</param>
        /// <param name="loop">Loop the sound.</param>
        public void playSound(string soundName, float volume, float pitch, float pan, bool loop)
        {
            SoundEffect sound = null;

            try
            {
                sound = UWB_Manager.mResources.Load<SoundEffect>("Content\\Sounds\\" + soundName);
                if (loop)
                    playSoundLooped(sound, volume, pitch, pan);
                else
                    playSound(sound, volume, pitch, pan);
            }
            catch (Exception e)
            {
                N_Utility.echoToStatusArea((e.Message));
            }
        }

        /// <summary>
        /// Stops a looping sound
        /// </summary>
        /// <param name="soundName">file name of sound you want to stop</param>
        public void stopLoopingSound(string soundName)
        {
            SoundEffect sound = null;

            try
            {
                sound = UWB_Manager.mResources.Load<SoundEffect>("Content\\Sounds\\" + soundName);
                mSoundEffectInstances[sound].Stop();
                mSoundEffectInstances.Remove(sound);
            }
            catch (Exception e)
            {
                N_Utility.echoToStatusArea((e.Message));
            }
        }

        /// <summary>
        /// Play a song (stops last playing song, loops by default)
        /// </summary>
        /// <param name="songName">The file name of the song you wish to play (should be in the Content\Songs folder).</param>
        public void playSong(string songName)
        {
            playSong(songName, true);
        }

        /// <summary>
        /// Play a song (stops last playing song)
        /// </summary>
        /// <param name="songName">The file name of the song you wish to play (should be in the Content\Songs folder).</param>
        /// <param name="loop">Repeat/loop the song or not.</param>
        public void playSong(string songName, bool loop)
        {
            Song song = null;
            
            try
            {
                song = UWB_Manager.mResources.Load<Song>("Content\\Songs\\" + songName);
                playSong(song, loop);
            }
            catch (Exception e)
            {
                N_Utility.echoToStatusArea((e.Message));
            }
        }

        /// <summary>
        /// Set the volume for song playing
        /// </summary>
        /// <param name="volume">volume to use</param>
        public void setSongVolume(float volume)
        {
            MediaPlayer.Volume = volume;
        }

        /// <summary>
        /// Mutes/Unmutes the currently playing song
        /// </summary>
        /// <param name="on"></param>
        public void muteSong(bool on)
        {
            MediaPlayer.IsMuted = on;
        }

        /// <summary>
        /// Play/Resume paused song
        /// </summary>
        public void playSong()
        {
            if (MediaPlayer.State == MediaState.Paused)
                MediaPlayer.Resume();
        }

        /// <summary>
        /// Pause the currently playing song
        /// </summary>
        public void pauseSong()
        {
            if (MediaPlayer.State == MediaState.Playing)
                MediaPlayer.Pause();
        }

        /// <summary>
        /// Stop currently playing song
        /// </summary>
        public void stopSong()
        {
            MediaPlayer.Stop();
        }

        /// <summary>
        /// Plays a sound based on specified volume, pitch, and panning.
        /// </summary>
        /// <param name="effect">The sound effect you wish to play.</param>
        /// <param name="volume">Volume, ranging from 0.0f (silence) to 1.0f (full volume). 1.0f is full volume relative to SoundEffect.MasterVolume.</param>
        /// <param name="pitch">Pitch adjustment, ranging from -1.0f (down one octave) to 1.0f (up one octave).  0.0f is unity (normal) pitch.</param>
        /// <param name="pan">Panning, ranging from -1.0f (full left) to 1.0f (full right). 0.0f is centered.</param>
        private void playSound(SoundEffect effect, float volume, float pitch, float pan)
        {
            if (effect != null)
            {
                FrameworkDispatcher.Update();
                effect.Play(volume, pitch, pan);
            }
        }

        /// <summary>
        /// Plays the sound looped.
        /// </summary>
        /// <param name="effect">effect to loop</param>
        /// <param name="volume">volume to use</param>
        /// <param name="pitch">pitch to use</param>
        /// <param name="pan">pan value to use</param>
        private void playSoundLooped(SoundEffect effect, float volume, float pitch, float pan)
        {
            if (effect != null)
            {
                FrameworkDispatcher.Update();
                SoundEffectInstance instance = effect.CreateInstance();
                instance.IsLooped = true;
                instance.Pan = pan;
                instance.Pitch = pitch;
                instance.Volume = volume;
                instance.Play();
                mSoundEffectInstances.Add(effect, instance);
            }
        }

        private void playSong(Song song, bool loop)
        {
            MediaPlayer.IsRepeating = loop;

            //If we are already playing a song then stop the last one
            if (MediaPlayer.State == MediaState.Playing)
                MediaPlayer.Stop();

            MediaPlayer.Play(song);
        }
    }
}
